//-------------------------------------------------------------------------------------------------
// Capsule2.hpp (engine)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#ifndef _include_Capsule2_
#define _include_Capsule2_


//-------------------------------------------------------------------------------------------------
// Included header files
//-------------------------------------------------------------------------------------------------
#include "Vector2.hpp"
#include "engineCommon.hpp"


///////////////////////////////////////////////////////////////////////////////////////////////////
//
//		Capsule2
//
///////////////////////////////////////////////////////////////////////////////////////////////////
class Capsule2
{
private:

	Vector2		_start;
	Vector2		_end;
	float		_radius;

public:

	/// Construction / Destruction
	~Capsule2(); // destructor
	Capsule2(); // default constructor
	Capsule2( const Capsule2& rhs ); // copy constructor
	Capsule2( const Vector2& startPoint, const Vector2& endPoint, float radius );

	/// Interface methods
	void SetStartEndRadius( const Vector2& startPoint, const Vector2& endPoint, float radius );
	void PushNewStartPoint( const Vector2& startPoint );
	void PushNewEndPoint( const Vector2& endPoint );
	void SetStart( const Vector2& startPoint );
	void SetEnd( const Vector2& endPoint );
	void SetStartEnd( const Vector2& startPoint, const Vector2& endPoint );
	inline void SetRadius( float radius ) { _radius = radius; }
	inline const Vector2& GetStart() const { return _start; }
	inline const Vector2& GetEnd() const { return _end; }
	inline float GetRadius() const { return _radius; }

//	bool DoMovingDiscsCollide( const Capsule2& compare );

	/// Operators
	const Capsule2& operator = ( const Capsule2& rhs );
	const Capsule2& operator += ( const Vector2& translation );
	const Capsule2 operator + ( const Vector2& translation ) const;
	const bool operator == ( const Capsule2& rhs ) const;
	const bool operator != ( const Capsule2& rhs ) const;
};

//-------------------------------------------------------------------------------------------------
// External Operators & Functions
//-------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------------------
// ~Capsule2() - destructor
//-------------------------------------------------------------------------------------------------
inline Capsule2::~Capsule2()
{
}


//-------------------------------------------------------------------------------------------------
// Capsule2() - default constructor
//-------------------------------------------------------------------------------------------------
inline Capsule2::Capsule2()
{
	// do nothing - keep this class as flyweight as possible
}


//-------------------------------------------------------------------------------------------------
// Capsule2( const Capsule2& Capsule2 ) - copy constructor
//-------------------------------------------------------------------------------------------------
inline Capsule2::Capsule2( const Capsule2& rhs )
:
_start( rhs._start ),
_end( rhs._end ),
_radius( rhs._radius )
{
}


//-------------------------------------------------------------------------------------------------
// Capsule2( vec2, vec2, float )
//-------------------------------------------------------------------------------------------------
inline Capsule2::Capsule2( const Vector2& startPoint, const Vector2& endPoint, float radius )
:
_start( startPoint ),
_end( endPoint ),
_radius( radius )
{
}


//-------------------------------------------------------------------------------------------------
// SetStartEndRadius
//-------------------------------------------------------------------------------------------------
inline void Capsule2::SetStartEndRadius( const Vector2& startPoint, const Vector2& endPoint, float radius )
{
	_start = startPoint;
	_end = endPoint;
	_radius = radius;
}


//-------------------------------------------------------------------------------------------------
// PushNewStartPoint
//-------------------------------------------------------------------------------------------------
inline void Capsule2::PushNewStartPoint( const Vector2& startPoint )
{
	_end = _start;
	_start = startPoint;
}


//-------------------------------------------------------------------------------------------------
// PushNewEndPoint
//-------------------------------------------------------------------------------------------------
inline void Capsule2::PushNewEndPoint( const Vector2& endPoint )
{
	_start = _end;
	_end = endPoint;
}


//-------------------------------------------------------------------------------------------------
// SetStart
//-------------------------------------------------------------------------------------------------
inline void Capsule2::SetStart( const Vector2& startPoint )
{
	_start = startPoint;
}


//-------------------------------------------------------------------------------------------------
// SetEnd
//-------------------------------------------------------------------------------------------------
inline void Capsule2::SetEnd( const Vector2& endPoint )
{
	_end = endPoint;
}


//-------------------------------------------------------------------------------------------------
// SetStartEnd
//-------------------------------------------------------------------------------------------------
inline void Capsule2::SetStartEnd( const Vector2& startPoint, const Vector2& endPoint )
{
	_start = startPoint;
	_end = endPoint;
}


//-------------------------------------------------------------------------------------------------
// operator =
//-------------------------------------------------------------------------------------------------
inline const Capsule2& Capsule2::operator = ( const Capsule2& rhs )
{
	if( this == &rhs )
		return *this;

	_start = rhs._start;
	_end = rhs._end;
	_radius = rhs._radius;
	return *this;
}


//-------------------------------------------------------------------------------------------------
// operator += (vec2)
//-------------------------------------------------------------------------------------------------
inline const Capsule2& Capsule2::operator += ( const Vector2& translation )
{
	_start += translation;
	_end += translation;
	return *this;
}


//-------------------------------------------------------------------------------------------------
// operator + (vec2)
//-------------------------------------------------------------------------------------------------
inline const Capsule2 Capsule2::operator + ( const Vector2& translation ) const
{
	Capsule2 translated( *this );
	translated += translation;
	return translated;
}




//-------------------------------------------------------------------------------------------------
// operator ==
//-------------------------------------------------------------------------------------------------
inline const bool Capsule2::operator == ( const Capsule2& rhs ) const
{
	return( (_start == rhs._start) && (_end == rhs._end) && (_radius == rhs._radius) );
}


//-------------------------------------------------------------------------------------------------
// operator !=
//-------------------------------------------------------------------------------------------------
inline const bool Capsule2::operator != ( const Capsule2& rhs ) const
{
	return( (_start != rhs._start) || (_end != rhs._end) || (_radius != rhs._radius) );
}


#endif // _include_Capsule2_
