//-------------------------------------------------------------------------------------------------
// Texture.cpp (engine)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#include "pch.hpp"
#include "Texture.hpp"
#include "engineAssert.hpp"


//-------------------------------------------------------------------------------------------------
// Class constants
//-------------------------------------------------------------------------------------------------
const dword_t Texture::TEXNUM_UNREGISTERED = 0xffffffff;


//-------------------------------------------------------------------------------------------------
// Constructor
//-------------------------------------------------------------------------------------------------
Texture::Texture( const std::string& filePath )
:
_width( 0 ),
_height( 0 ),
_bytesPerPixel( 0 ),
_internalFormat( 0 ),
_glTexNum( TEXNUM_UNREGISTERED )
{
	Image image( filePath );
	_InitFromImage( image );
	_CreateOpenGLTexture( image );
}


//-------------------------------------------------------------------------------------------------
// Constructor
//-------------------------------------------------------------------------------------------------
Texture::Texture( const Image& image )
:
_width( 0 ),
_height( 0 ),
_bytesPerPixel( 0 ),
_internalFormat( 0 ),
_glTexNum( TEXNUM_UNREGISTERED )
{
	_InitFromImage( image );
	_CreateOpenGLTexture( image );
}


//-------------------------------------------------------------------------------------------------
// Destructor
//-------------------------------------------------------------------------------------------------
Texture::~Texture()
{
}


//-------------------------------------------------------------------------------------------------
// InitFromImage
//-------------------------------------------------------------------------------------------------
void Texture::_InitFromImage( const Image& image )
{
	_width = image._width;
	_height = image._height;
	_bytesPerPixel = image._bytesPerPixel;
	_internalFormat = image._internalFormat;
}


//-------------------------------------------------------------------------------------------------
// CreateOpenGLTexture
//-------------------------------------------------------------------------------------------------
void Texture::_CreateOpenGLTexture( const Image& image )
{
	/// Create an opengl texture name (number) for this image
	_glTexNum = 0;
	glPixelStorei( GL_UNPACK_ALIGNMENT, 1 ); // pixel info is byte-aligned
	glGenTextures( 1, &_glTexNum ); // create a texnum for this new image
	glBindTexture( GL_TEXTURE_2D, _glTexNum ); // select this as the current 2d texture
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );

	/// Upload image data to OpenGL
	glTexImage2D( 
		GL_TEXTURE_2D,			// creating a 2d texture
		0,						// use the root picmip level for this texture (0 = best)
		_internalFormat,		// format opengl should use internally for this image
		_width,					// width of the source data, in pixels
		_height,				// height of the source data, in pixels
		0,						// do not add a border
		image._bufferFormat,	// the layout of the pixels in the buffer
		GL_UNSIGNED_BYTE,		// pixel color components are unsigned bytes
		image._pixelData );		// buffer containing pixel data bytes
}



