//-------------------------------------------------------------------------------------------------
// winMain.cpp (engine)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
// 
// Implementation file for the Win32 application entry point, WinMain(), and
//	the main Win32 message-handling function, WinProc().
//-------------------------------------------------------------------------------------------------
#include "pch.hpp"
#include "engineCommon.hpp"
#include "TheApplication.hpp"
#include "TheMessageHandler.hpp"


//-------------------------------------------------------------------------------------------------
// WinProc
// 
// The main Win32 message handling callback function for this program.
// Any time a new message (WM_SIZE, WM_QUIT, etc.) arrives, windows calls this function.
//-------------------------------------------------------------------------------------------------
LRESULT CALLBACK WinProc(	HWND	hWnd,		// handle for this window
							UINT	uMsg,		// the type of message being received
							WPARAM	wParam,		// additional message data (optional)
							LPARAM	lParam )	// additional message data (optional)
{
	bool			messageHandled; // true if message was handled internally
	LRESULT			retVal = 0;

	/// Let the application handle this message; if it doesn't, send it to the default handler
	messageHandled = theMessageHandler.HandleWin32Message( hWnd, uMsg, wParam, lParam );
	if( !messageHandled )
	{
		retVal = DefWindowProc( hWnd, uMsg, wParam, lParam );
	}

	return( retVal );
}


//-------------------------------------------------------------------------------------------------
// RunMessagePumpLoop
// 
// A traditional windows application message pump loop.
// Returns only when the application has finished and is in the process of shutting down.
//-------------------------------------------------------------------------------------------------
void RunMessagePumpLoop()
{
	MSG msg; // Windows message structure
	memset( &msg, 0, sizeof( msg ) );

	/// Loop forever until the application indicates that it is shutting down
	while( !theApplication.IsExiting() )
	{
		if( PeekMessage( &msg, NULL, 0, 0, PM_REMOVE ) ) // are there any messages waiting?
		{
			if( msg.message == WM_QUIT ) // was it a QUIT message?
			{
				/// Inform the application
				theApplication.OnQuitMessage();
				continue;
			}

			/// Pass this message on to window message processing
			TranslateMessage( &msg ); // translate virtual-key message into character-message
			DispatchMessage( &msg ); // and send it on its merry way to WinProc
		}
		else // no messages in queue
		{
			Sleep( 1 ); // this makes the program more friendly for alt-tabbing & multiprocessing

			/// Instruct the application to run a single frame
			theApplication.RunFrame();
		}
	}
}



//-------------------------------------------------------------------------------------------------
// WinMain
// 
// The Win32 application entry function.  Everything starts here.
//-------------------------------------------------------------------------------------------------
int WINAPI WinMain(	HINSTANCE	hInstance,		// handle to this instance of the application
					HINSTANCE	hPrevInstance,	// legacy (unused)
					LPSTR		lpCmdLine,		// command line arguments
					int			nCmdShow )		// initial window show-state
{
	/// Make the compiler happy
	UNUSED_ARG( hPrevInstance );
	UNUSED_ARG( nCmdShow );

	/// Create the application singleton
	theApplication.SetHInstance( hInstance );
	theApplication.SetCmdLine( lpCmdLine );

	/// Initialize the application and its various subsystems
	theApplication.Startup();

	/// Run the main windows message pump; this will return only when the application is finished
	RunMessagePumpLoop();

	return( TheApplication::EXIT_MESSAGE_NORMAL ); // exit the program
}
