//-------------------------------------------------------------------------------------------------
// Animation.cpp (game)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#include "pch.hpp"
#include "TheGame.hpp"
#include "Animation.hpp"


/*
-----------------------------------------------------------------
HOW IT WORKS (The Querks)
---------------------
The animation class makes the folowing assumptions.
* All rectangular images are animations (width / height = frames)
* If the Constructor hits any magenta on the bottom horizontal line of the image while searching through the anim, it assumes the anims end there and there are no more in the set

-----------------------------------------------------------------
*/



//-------------------------------------------------------------------------------------------------
// Destructor
//-------------------------------------------------------------------------------------------------
Animation::~Animation()
{}


//-------------------------------------------------------------------------------------------------
// Constructor
// TAKE: (File Path, Frame Rate [per sec], scale, rotation)
// DESC: Create a sprite
//-------------------------------------------------------------------------------------------------
Animation::Animation( const std::string& filePath, float aFrate, const Vector2& aScale, float aRotDeg )
{

	a_texture = NULL;

	Image testImage( filePath );
	
	a_texture = new Texture( testImage );
	VALIDATE( a_texture );

	if ( !a_texture )		return ;
	
	size = a_texture->GetHeight();
	totalframes = a_texture->GetWidth() / size;
	framenum = 0;
	starttime = theApplication.GetTimeFrameBegan();
	frate = aFrate;
	scale = aScale;
	rotdeg = aRotDeg;

	//----------------------------------
	// Scan Image for solid blocks of Magenta, if one is found, assume no more animations in this set

	if (totalframes > 1)
	{
		unsigned char*	pixelData	= testImage.GetRawPixelData() ;
		unsigned char	pixelB		= 0;
		unsigned char	pixelG		= 0;
		unsigned char	pixelR		= 0;
		unsigned char	pixelA		= 0;
		unsigned int	pixel		= 0;
		unsigned int	width		= size;
		unsigned int	end			= 0;
		int				realframes	= 0;


		unsigned int x = 0;

		for ( int i = 0; i < totalframes; i++)
		{
			x = i * size;
			width = (i+1) * size;

			for ( x; x < width; ++x )
			{
				unsigned imagePos = ( x ) * 4 ;
				pixelB		= pixelData[ imagePos + 0 ];
				pixelG		= pixelData[ imagePos + 1 ];
				pixelR		= pixelData[ imagePos + 2 ];
				pixelA		= pixelData[ imagePos + 3 ];

//				pixel		= (0 << 24) + (pixelR << 16) + (pixelG << 8) + (pixelB << 0);
//				if (pixel == 0x00ff00ff)
				if( pixelR == 0xff && pixelG == 0x00 && pixelB == 0xff )
				{
					end = 1;
					break;
				}
			}

			if (end)
				break;
			else
				realframes++;

		}

		totalframes = realframes;
	}

}




//-------------------------------------------------------------------------------------------------
// Draw()
// TAKE: (Screen Coord, scale, rotation, frame to draw)
// DESC: Draw a sprite on the screen
//-------------------------------------------------------------------------------------------------
void Animation::Draw( const Vector2& screenCenterCoordinates, const Vector2& aScale, float aRotDeg, const Rgba& color, int aDrawFrame )
{
	// Check Draw Frame
	if (aDrawFrame != -666)
	{
		framenum = aDrawFrame;
		loopcount = 0;
	}
	// Define coordinates of our image w/in the texture
	Vector2 topleft(0,0),botright(1,1);
	if (totalframes > 1)
	{
		// Top Coord
		if (framenum != 0)
			topleft.x = (float)(size * framenum) / (float)a_texture->GetWidth();
		else
			topleft.x = 0;

		topleft.y = 0;

		// Bottom Coord
		botright.x = topleft.x + ((float)size / (float)a_texture->GetWidth());

		botright.y = 1;

	}
	
	Vector2 rectsize( (float)size, (float)size );


	/// draw using the renderer and texture coordinates
	theRenderer.DrawTexturedRectCentered( *a_texture, screenCenterCoordinates, rectsize, topleft, botright, aRotDeg, aScale, color );

	// update Frame tick /
	double timepassed;
	timepassed = theApplication.GetTimeFrameBegan() - starttime;
	if ((timepassed >= (1 / frate)) && totalframes > 1)
	{
		framenum++;
		if (framenum > (totalframes - 1))
		{
				// increment loop / reset frame
				loopcount++;
				framenum = 0;
		}

		// reset frame time
		starttime = theApplication.GetTimeFrameBegan();

	}



}

//-------------------------------------------------------------------------------------------------
// GetLoop()
// returns the number of loop iterations that have occured
//-------------------------------------------------------------------------------------------------
int Animation::GetLoop()
{
	return loopcount;
}

//-------------------------------------------------------------------------------------------------
// ResetAnim()
// Reset loop count and frame number
//-------------------------------------------------------------------------------------------------
void Animation::ResetAnim()
{
	loopcount = 0;
	framenum = 0;
}