//-------------------------------------------------------------------------------------------------
// Entity.cpp
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
// 
// Master entity file.  Put no entity-specific code here.
//-------------------------------------------------------------------------------------------------
#include "pch.hpp"
#include "engineAssert.hpp"
#include "Entity.hpp"
#include "TheGame.hpp"


//-------------------------------------------------------------------------------------------------
// constructor
//-------------------------------------------------------------------------------------------------
Entity::Entity( const TypeOfEntity type )
:
m_type( type ),
m_isEnemy( false ),
m_stunned( false ),
m_isHitByPlayerBullets( false ),
m_isBeingDamagedByPlayer( false ),
m_bossStage( 0 ),
m_pos( 0, 0 ),
m_vel( 0, 0 ),
m_rot( 0 ),
m_anim( 0 ),
m_readyToDie( false ),
m_damageGiveMultiplier( 0.0f ),
m_damageTakeMultiplier( 0.0f ),
m_hitPoints( 100.0f ),
m_drawRotated( true ),
m_scale( Vector2::ONE ),
m_color( Rgba::WHITE ),
m_birthTime( theApplication.GetTimeFrameBegan() ),
m_timeOfLastSpawn( 0.0 ),
m_genericCounter( 0 ),
m_genericTimeStamp( 0.0 ),
m_genericTimeStamp2( 0.0 )
{
}


//-------------------------------------------------------------------------------------------------
// Think
//-------------------------------------------------------------------------------------------------
void Entity::Think()
{
	/// Run the appropriate think function
	switch( m_type )
	{
		case ENT_PLAYER_START:				Think_PlayerStart();			break;
		case ENT_BARREL:					Think_Barrel();					break;
		case ENT_CRATE:						Think_Crate();					break;
		case ENT_BOMB:						Think_Bomb();					break;
		case ENT_STUNNER:					Think_Stunner();				break;
		case ENT_ITEMPICKUP_BOMB:			Think_ItemPickupBomb();			break;
		case ENT_ITEMPICKUP_HEALTH:			Think_ItemPickupHealth();		break;
		case ENT_ITEMPICKUP_INVISIBILITY:	Think_ItemPickupInvisibility();	break;
		case ENT_ITEMPICKUP_KEY:			Think_ItemPickupKey();			break;
		case ENT_ITEMPICKUP_MANA:			Think_ItemPickupMana();			break;
		case ENT_ITEMPICKUP_STUNNER:		Think_ItemPickupStunner();		break;
		case ENT_ENEMY_BOSS1:				Think_EnemyBoss1();				break;
		case ENT_ENEMY_BOSS2:				Think_EnemyBoss2();				break;
		case ENT_ENEMY_GRUNT:				Think_EnemyGrunt();				break;
		case ENT_ENEMY_MUSHROOM:			Think_EnemyMushroom();			break;
		case ENT_ENEMY_WIZROBE:				Think_EnemyWizrobe();			break;
		case ENT_BULLET_WIZROBE:			Think_BulletWizrobe();			break;
		case ENT_BULLET_BOSS1:				Think_BulletBoss1();			break;
		case ENT_BULLET_BOSS2:				Think_BulletBoss2();			break;
		case ENT_GENERATOR_GRUNT:			Think_GeneratorGrunt();			break;
		case ENT_GENERATOR_MUSHROOM:		Think_GeneratorMushroom();		break;
		case ENT_GENERATOR_WIZROBE:			Think_GeneratorWizrobe();		break;
		case ENT_GENERATOR_BOSS1:			Think_GeneratorBoss1();			break;
		case ENT_GENERATOR_SINGLEBOSS1:		Think_GeneratorSingleBoss1();	break;
		case ENT_GENERATOR_BOSS2:			Think_GeneratorBoss2();			break;
		case ENT_GENERATOR_SINGLEBOSS2:		Think_GeneratorSingleBoss2();	break;
		case ENT_PLAYER_SPELL1:				Think_PlayerSpell1();			break;
		case ENT_PLAYER_SPELL2:				Think_PlayerSpell2();			break;
		case ENT_PLAYER_SPELL3:				Think_PlayerSpell3();			break;
	};
}

//-------------------------------------------------------------------------------------------------
// SpawnFunc
//-------------------------------------------------------------------------------------------------
void Entity::SpawnFunc()
{
	/// Run the appropriate spawn function
	switch( m_type )
	{
		case ENT_PLAYER_START:				Spawn_PlayerStart();			break;
		case ENT_BARREL:					Spawn_Barrel();					break;
		case ENT_CRATE:						Spawn_Crate();					break;
		case ENT_BOMB:						Spawn_Bomb();					break;
		case ENT_STUNNER:					Spawn_Stunner();				break;
		case ENT_ITEMPICKUP_BOMB:			Spawn_ItemPickupBomb();			break;
		case ENT_ITEMPICKUP_HEALTH:			Spawn_ItemPickupHealth();		break;
		case ENT_ITEMPICKUP_INVISIBILITY:	Spawm_ItemPickupInvisibility();	break;
		case ENT_ITEMPICKUP_KEY:			Spawn_ItemPickupKey();			break;
		case ENT_ITEMPICKUP_MANA:			Spawn_ItemPickupMana();			break;
		case ENT_ITEMPICKUP_STUNNER:		Spawn_ItemPickupStunner();		break;
		case ENT_ENEMY_BOSS1:				Spawn_EnemyBoss1();				break;
		case ENT_ENEMY_BOSS2:				Spawn_EnemyBoss2();				break;
		case ENT_ENEMY_GRUNT:				Spawn_EnemyGrunt();				break;
		case ENT_ENEMY_MUSHROOM:			Spawn_EnemyMushroom();			break;
		case ENT_ENEMY_WIZROBE:				Spawn_EnemyWizrobe();			break;
		case ENT_BULLET_WIZROBE:			Spawn_BulletWizrobe();			break;
		case ENT_BULLET_BOSS1:				Spawn_BulletBoss1();			break;
		case ENT_BULLET_BOSS2:				Spawn_BulletBoss2();			break;
		case ENT_GENERATOR_GRUNT:			Spawn_GeneratorGrunt();			break;
		case ENT_GENERATOR_MUSHROOM:		Spawn_GeneratorMushroom();		break;
		case ENT_GENERATOR_WIZROBE:			Spawn_GeneratorWizrobe();		break;
		case ENT_GENERATOR_BOSS1:			Spawn_GeneratorBoss1();			break;
		case ENT_GENERATOR_SINGLEBOSS1:		Spawn_GeneratorSingleBoss1();	break;
		case ENT_GENERATOR_BOSS2:			Spawn_GeneratorBoss2();			break;
		case ENT_GENERATOR_SINGLEBOSS2:		Spawn_GeneratorSingleBoss2();	break;
		case ENT_PLAYER_SPELL1:				Spawn_PlayerSpell1();			break;
		case ENT_PLAYER_SPELL2:				Spawn_PlayerSpell2();			break;
		case ENT_PLAYER_SPELL3:				Spawn_PlayerSpell3();			break;
	};
}


//-------------------------------------------------------------------------------------------------
// destructor
//-------------------------------------------------------------------------------------------------
Entity::~Entity()
{
	/// Run the appropriate death function
	switch( m_type )
	{
		case ENT_PLAYER_START:				Die_PlayerStart();				break;
		case ENT_BARREL:					Die_Barrel();					break;
		case ENT_CRATE:						Die_Crate();					break;
		case ENT_BOMB:						Die_Bomb();						break;
		case ENT_STUNNER:					Die_Stunner();					break;
		case ENT_ITEMPICKUP_BOMB:			Die_ItemPickupBomb();			break;
		case ENT_ITEMPICKUP_HEALTH:			Die_ItemPickupHealth();			break;
		case ENT_ITEMPICKUP_INVISIBILITY:	Die_ItemPickupInvisibility();	break;
		case ENT_ITEMPICKUP_KEY:			Die_ItemPickupKey();			break;
		case ENT_ITEMPICKUP_MANA:			Die_ItemPickupMana();			break;
		case ENT_ITEMPICKUP_STUNNER:		Die_ItemPickupStunner();		break;
		case ENT_ENEMY_BOSS1:				Die_EnemyBoss1();				break;
		case ENT_ENEMY_BOSS2:				Die_EnemyBoss2();				break;
		case ENT_ENEMY_GRUNT:				Die_EnemyGrunt();				break;
		case ENT_ENEMY_MUSHROOM:			Die_EnemyMushroom();			break;
		case ENT_ENEMY_WIZROBE:				Die_EnemyWizrobe();				break;
		case ENT_BULLET_WIZROBE:			Die_BulletWizrobe();			break;
		case ENT_BULLET_BOSS1:				Die_BulletBoss1();				break;
		case ENT_BULLET_BOSS2:				Die_BulletBoss2();				break;
		case ENT_GENERATOR_GRUNT:			Die_GeneratorGrunt();			break;
		case ENT_GENERATOR_MUSHROOM:		Die_GeneratorMushroom();		break;
		case ENT_GENERATOR_WIZROBE:			Die_GeneratorWizrobe();			break;
		case ENT_GENERATOR_BOSS1:			Die_GeneratorBoss1();			break;
		case ENT_GENERATOR_SINGLEBOSS1:		Die_GeneratorSingleBoss1();		break;
		case ENT_GENERATOR_BOSS2:			Die_GeneratorBoss2();			break;
		case ENT_GENERATOR_SINGLEBOSS2:		Die_GeneratorSingleBoss2();		break;
		case ENT_PLAYER_SPELL1:				Die_PlayerSpell1();				break;
		case ENT_PLAYER_SPELL2:				Die_PlayerSpell2();				break;
		case ENT_PLAYER_SPELL3:				Die_PlayerSpell3();				break;
	};
}


//-------------------------------------------------------------------------------------------------
// DealDamage
//-------------------------------------------------------------------------------------------------
void Entity::DealDamage( float damageAmount )
{
	float damageDealt = m_damageTakeMultiplier * damageAmount;
	if( damageDealt > 0.0f )
	{
		m_hitPoints -= damageDealt;
		if( m_hitPoints <= 0.0f )
		{
			m_readyToDie = true;
		}
	}
}

//-------------------------------------------------------------------------------------------------
// SpawnNewEntity
//-------------------------------------------------------------------------------------------------
STATIC Entity& Entity::SpawnNewEntity( const TypeOfEntity type, const Vector2& pos, const double rot )
{
	Entity* e = new Entity( type );
	VALIDATE( e );

	e->m_pos = pos;
	e->m_rot = rot;
	MapData& mapData = *theGame->m_currentMap;
	mapData.entityList.push_back( e );
	e->SpawnFunc();

	return *e;
}

//-------------------------------------------------------------------------------------------------
// SpawnNewEntity
//-------------------------------------------------------------------------------------------------
STATIC Entity& Entity::SpawnNewEntity( const TypeOfEntity type, const Vector2& pos )
{
	Entity* e = new Entity( type );
	VALIDATE( e );

	e->m_pos = pos;
	MapData& mapData = *theGame->m_currentMap;
	mapData.entityList.push_back( e );
	e->SpawnFunc();

	return *e;
}

//-------------------------------------------------------------------------------------------------
// SpawnNewEntity
//-------------------------------------------------------------------------------------------------
STATIC Entity& Entity::SpawnNewEntity( const TypeOfEntity type )
{
	Entity* e = new Entity( type );
	VALIDATE( e );

	MapData& mapData = *theGame->m_currentMap;
	mapData.entityList.push_back( e );
	e->SpawnFunc();

	return *e;
}

//-------------------------------------------------------------------------------------------------
// UpdateEntities
//-------------------------------------------------------------------------------------------------
STATIC void Entity::UpdateEntities()
{
	MapData& mapData = *theGame->m_currentMap;
	std::list< Entity* >& entList = mapData.entityList;
	std::list< Entity* >::iterator iter;
	std::list< Entity* >::iterator last;
	for( iter = entList.begin(); iter != entList.end(); )
	{
		last = iter;
		++ iter;

		Entity* ent = *last;

		/// Check for death
		if( ent->m_readyToDie )
		{
			delete ent;
			entList.erase( last );
			continue;
		}

		ent->Think();
	}
}


//-------------------------------------------------------------------------------------------------
// TryToMove
//-------------------------------------------------------------------------------------------------
bool Entity::TryToMove( const Vector2& moveVector )
{
	Vector2 newPos = m_pos + moveVector;
	Vector2 newWorldMins = (newPos * 32.0f) + m_bounds.mins;
	Vector2 newWorldMaxs = (newPos * 32.0f) + m_bounds.maxs;
	AABB2 newWorldBounds( newWorldMins, newWorldMaxs );

	bool isOkayToMove = !theGame->DoBoundsCollideWithMap( newWorldBounds, MATFLAG_BLOCKS_ENEMIES );
	if( !isOkayToMove )
	{
		return false;
	}

	m_pos = newPos;

	return true;
}

