//-------------------------------------------------------------------------------------------------
// Entity.hpp
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
// 
// Master header file for all entities.
//-------------------------------------------------------------------------------------------------
#pragma once
#include "Animation.hpp"
//#include "TheGame.hpp"
#include "../Engine/AABB2.hpp"


enum TypeOfEntity {
	ENT_NONE,
	ENT_PLAYER_START,
	ENT_BARREL,
	ENT_CRATE,
	ENT_BOMB,
	ENT_STUNNER,
	ENT_ITEMPICKUP_BOMB,
	ENT_ITEMPICKUP_HEALTH,
	ENT_ITEMPICKUP_INVISIBILITY,
	ENT_ITEMPICKUP_KEY,
	ENT_ITEMPICKUP_MANA,
	ENT_ITEMPICKUP_STUNNER,
	ENT_ENEMY_GRUNT,
	ENT_ENEMY_MUSHROOM,
	ENT_ENEMY_WIZROBE,
	ENT_ENEMY_BOSS1,
	ENT_ENEMY_BOSS2,
	ENT_BULLET_WIZROBE,
	ENT_BULLET_BOSS1,
	ENT_BULLET_BOSS2,
	ENT_GENERATOR_GRUNT,
	ENT_GENERATOR_MUSHROOM,
	ENT_GENERATOR_WIZROBE,
	ENT_GENERATOR_BOSS1,
	ENT_GENERATOR_BOSS2,
	ENT_GENERATOR_SINGLEBOSS1,
	ENT_GENERATOR_SINGLEBOSS2,
	ENT_PLAYER_SPELL1,
	ENT_PLAYER_SPELL2,
	ENT_PLAYER_SPELL3,
};


extern std::map< std::string, TypeOfEntity > g_entityTypeMap;


class Entity
{
public:

	static void UpdateEntities();

	Entity( const TypeOfEntity type = ENT_NONE );
	~Entity();

	//-------------------------------------------------------------------------------------------------
	// NOTE: This is going to be a mega-monolithic catch-all class.  Suck it down.  Three hours left.
	//-------------------------------------------------------------------------------------------------
	TypeOfEntity	m_type;
	bool			m_isEnemy;
	bool			m_stunned;
	bool			m_isHitByPlayerBullets;
	bool			m_isBeingDamagedByPlayer;
	Vector2			m_pos;		// position in tile coordinates
	Vector2			m_vel;		// velocity in tile coordinates per second
	double			m_rot;		// rotation in degrees
	double			m_rotVel;	// rotational velocity
	Animation*		m_anim;		// animation, can be NULL
	AABB2			m_bounds;
	bool			m_readyToDie;
	float			m_damageGiveMultiplier;
	float			m_damageTakeMultiplier;
	float			m_hitPoints;
	bool			m_drawRotated;
	Vector2			m_scale;
	Rgba			m_color;
	float			m_hurtColorFade;
	int				m_bossStage;

	double			m_objectSpawnPeriodSeconds; // number of seconds between spawns at maximum speed
	double			m_timeOfLastSpawn;
	double			m_birthTime;
	TypeOfEntity	m_objectSpawnType;
	unsigned int	m_genericStateEnum;
	double			m_genericTimeStamp;
	double			m_genericTimeStamp2; // Good lord, this monolithic entity class was a BAD IDEA - next time, take 5 minutes to set up some polymorphism
	unsigned int	m_genericCounter;

	bool TryToMove( const Vector2& moveVector );
	void Think();
	static Entity& SpawnNewEntity( const TypeOfEntity type, const Vector2& pos , const double rot );
	static Entity& SpawnNewEntity( const TypeOfEntity type, const Vector2& pos );
	static Entity& SpawnNewEntity( const TypeOfEntity type );
	void SpawnFunc();
	void DealDamage( float damageAmount );

	//-------------------------------------------------------------------------------------------------
	// Various entity methods
	//-------------------------------------------------------------------------------------------------

	/// Entity_Barrel.cpp
	void Spawn_Barrel();
	void Think_Barrel();
	void Die_Barrel();

	/// Entity_Crate.cpp
	void Spawn_Crate();
	void Think_Crate();
	void Die_Crate();

	/// Entity_Bomb.cpp
	void Spawn_Bomb();
	void Think_Bomb();
	void Die_Bomb();

	/// Entity_Stunner.cpp
	void Spawn_Stunner();
	void Think_Stunner();
	void Die_Stunner();

	/// Entity_ItemPickupBomb.cpp
	void Spawn_ItemPickupBomb();
	void Think_ItemPickupBomb();
	void Die_ItemPickupBomb();

	/// Entity_ItemPickupHealth.cpp
	void Spawn_ItemPickupHealth();
	void Think_ItemPickupHealth();
	void Die_ItemPickupHealth();

	/// Entity_ItemPickupInvisibility.cpp
	void Spawm_ItemPickupInvisibility();
	void Think_ItemPickupInvisibility();
	void Die_ItemPickupInvisibility();

	/// Entity_ItemPickupKey.cpp
	void Spawn_ItemPickupKey();
	void Think_ItemPickupKey();
	void Die_ItemPickupKey();

	/// Entity_ItemPickupMana.cpp
	void Spawn_ItemPickupMana();
	void Think_ItemPickupMana();
	void Die_ItemPickupMana();

	/// Entity_ItemPickupStunner.cpp
	void Spawn_ItemPickupStunner();
	void Think_ItemPickupStunner();
	void Die_ItemPickupStunner();

	/// Entity_GeneratorGrunt.cpp
	void Spawn_GeneratorGrunt();
	void Think_GeneratorGrunt();
	void Die_GeneratorGrunt();

	/// Entity_GeneratorMushroom.cpp
	void Spawn_GeneratorMushroom();
	void Think_GeneratorMushroom();
	void Die_GeneratorMushroom();

	/// Entity_GeneratorWizrobe.cpp
	void Spawn_GeneratorWizrobe();
	void Think_GeneratorWizrobe();
	void Die_GeneratorWizrobe();

	/// Entity_GeneratorBoss1.cpp
	void Spawn_GeneratorBoss1();
	void Think_GeneratorBoss1();
	void Die_GeneratorBoss1();
	void Spawn_GeneratorSingleBoss1();
	void Think_GeneratorSingleBoss1();
	void Die_GeneratorSingleBoss1();

	/// Entity_GeneratorBoss2.cpp
	void Spawn_GeneratorBoss2();
	void Think_GeneratorBoss2();
	void Die_GeneratorBoss2();
	void Spawn_GeneratorSingleBoss2();
	void Think_GeneratorSingleBoss2();
	void Die_GeneratorSingleBoss2();

	/// Entity_EnemyBoss1.cpp
	void Spawn_EnemyBoss1();
	void Think_EnemyBoss1();
	void Die_EnemyBoss1();
	void Boss1Shoot1();
	void Boss1Shoot2();

	/// Entity_EnemyBoss2.cpp
	void Spawn_EnemyBoss2();
	void Think_EnemyBoss2();
	void Die_EnemyBoss2();
	void Boss2Shoot1();
	
	/// Entity_EnemyGrunt.cpp
	void Spawn_EnemyGrunt();
	void Think_EnemyGrunt();
	void Die_EnemyGrunt();

	/// Entity_EnemyMushroom.cpp
	void Spawn_EnemyMushroom();
	void Think_EnemyMushroom();
	void Die_EnemyMushroom();
	void MushroomShoot();

	/// Entity_EnemyWizrobe.cpp
	void Spawn_EnemyWizrobe();
	void Think_EnemyWizrobe();
	void Die_EnemyWizrobe();
	void WizrobeShoot();

	/// Entity_BulletWizrobe.cpp
	void Spawn_BulletWizrobe();
	void Think_BulletWizrobe();
	void Die_BulletWizrobe();

	/// Entity_BulletBoss1.cpp
	void Spawn_BulletBoss1();
	void Think_BulletBoss1();
	void Die_BulletBoss1();

	/// Entity_BulletBoss2.cpp
	void Spawn_BulletBoss2();
	void Think_BulletBoss2();
	void Die_BulletBoss2();

	/// Entity_PlayerSpell1.cpp
	void Spawn_PlayerSpell1();
	void Think_PlayerSpell1();
	void Die_PlayerSpell1();

	/// Entity_PlayerSpell2.cpp
	void Spawn_PlayerSpell2();
	void Think_PlayerSpell2();
	void Die_PlayerSpell2();

	/// Entity_PlayerSpell2.cpp
	void Spawn_PlayerSpell3();
	void Think_PlayerSpell3();
	void Die_PlayerSpell3();

	/// Entity_PlayerStart.cpp
	void Spawn_PlayerStart();
	void Think_PlayerStart();
	void Die_PlayerStart();
};
