//-------------------------------------------------------------------------------------------------
// Entity_Bomb.cpp
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#include "pch.hpp"
#include "engineAssert.hpp"
#include "Entity.hpp"
#include "Particle.hpp"
#include "TheGame.hpp"


const double BOMB_LIFESPAN_SECONDS = 1.5;
const float BOMB_EXPLOSION_ENEMY_RADIUS = 5.0f;
const float BOMB_EXPLOSION_MAP_RADIUS = 3.0f;
const float BOMB_EXPLOSION_VISUAL_RADIUS = 4.0f;
const float BOMB_EXPLOSION_MAX_DAMAGE = 30.0f;


//-------------------------------------------------------------------------------------------------
// Spawn_Bomb
//-------------------------------------------------------------------------------------------------
void Entity::Spawn_Bomb()
{
	theConsole.Printf( "Spawned a Bomb at %.02f,%.02f\n", m_pos.x, m_pos.y );
	m_anim = new Animation( "data/gfx/Items/bomb_ticking.tga" );
	m_bounds.SetFromMinXYMaxXY( -8.0f, -8.0f, 8.0f, 8.0f );
	m_rotVel = 0.0f;
}


//-------------------------------------------------------------------------------------------------
// Think_Bomb
//-------------------------------------------------------------------------------------------------
void Entity::Think_Bomb()
{
	double timeNow = theApplication.GetTimeFrameBegan();
	double age = timeNow - m_birthTime;

	if( age > BOMB_LIFESPAN_SECONDS )
	{
		m_readyToDie = true;
	}
}


//-------------------------------------------------------------------------------------------------
// Die_Bomb
//-------------------------------------------------------------------------------------------------
void Entity::Die_Bomb()
{
	delete m_anim ;

	Particle* p = Particle::Spawn();
	p->m_pos = m_pos;
	p->m_startAlpha = 1.0f;
	p->m_endAlpha = 0.0f;
	p->m_startScale = 0.0f;
	p->m_endScale = BOMB_EXPLOSION_VISUAL_RADIUS;
	p->m_timeToLive = 0.35;
	p->m_anim = new Animation( "data/gfx/particles/BombExplode.tga" );

	DamageMapGridRadius( m_pos, BOMB_EXPLOSION_MAP_RADIUS , true );
	float bombDamage = BOMB_EXPLOSION_MAX_DAMAGE * ( theGame->m_player->m_difficulty * 0.25f );
	if( bombDamage < BOMB_EXPLOSION_MAX_DAMAGE )
	{
		bombDamage = BOMB_EXPLOSION_MAX_DAMAGE;
	}
	DamageEntitiesRadius( m_pos, BOMB_EXPLOSION_ENEMY_RADIUS, bombDamage );

	/// Check if we hit the player
	if( theGame->IsPointWithinGivenRadiusOfPlayer( m_pos, BOMB_EXPLOSION_MAP_RADIUS ) )
	{
		theGame->m_player->AdjustHealth( -bombDamage * 0.5f );
	}
}



