//-------------------------------------------------------------------------------------------------
// Entity_BulletBoss1.cpp
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#include "pch.hpp"
#include "engineAssert.hpp"
#include "Entity.hpp"
#include "Particle.hpp"
#include "TheGame.hpp"


const float BULLETBOSS1_MOVE_TILES_PER_SECOND = 8.0;
const double BULLETBOSS1_LIFESPAN_SECONDS = 1.0;
const float BULLETBOSS1_DAMAGE = 5.0f;


//-------------------------------------------------------------------------------------------------
void Entity::Spawn_BulletBoss1()
{
	m_anim = new Animation( "data/gfx/Spells/Spell06.tga" );
	m_bounds.SetFromMinXYMaxXY( -8.0f, -8.0f, 8.0f, 8.0f );
	m_rotVel = RandomFloat( -10.0f, 10.0f );
	m_hitPoints = 10.0f;
	m_drawRotated = true;
	m_scale.SetXY( .25, .25 );
	m_color = Rgba( 255, 255, 255, 255 );
	m_damageGiveMultiplier = theGame->m_player->m_difficulty * 0.2f;	
	if( m_damageGiveMultiplier < 1 )
	{
		m_damageGiveMultiplier = 1;
	}
	m_damageTakeMultiplier = 0.0f;

	/// Calculate a normal vector towards the player
	Vector2 vectorTowardPlayer = theGame->m_player->pos - m_pos;
	if( theGame->m_player->IsInvisible() )
	{
		m_rot = RandomFloat( 0.0f, 360.0f );
	}
	else
	{
		m_rot = vectorTowardPlayer.GetDegrees();
		m_rot += RandomFloat( -15.0f, 15.0f );
	}
	m_vel.SetFromCosSinDegrees( m_rot );
}


//-------------------------------------------------------------------------------------------------
void Entity::Think_BulletBoss1()
{
	double timeNow = theApplication.GetTimeFrameBegan();
	double age = timeNow - m_birthTime;

	float moveSpeed = (float)( BULLETBOSS1_MOVE_TILES_PER_SECOND * theApplication.GetFrameSeconds() );
	Vector2 moveVector = m_vel * moveSpeed;
	m_scale.x = m_scale.x + 0.05f;
	m_scale.y = m_scale.y + 0.05f;
	m_color.a = (unsigned char)( m_color.a * 0.9f );
	m_rot += m_rotVel;

	bool success = TryToMove( moveVector );
	if( !success )
	{
		/// Impacted on the world
		m_readyToDie = true;
	}

	/// Check if we hit the player
	if( theGame->m_player->m_worldBounds.IsPointInsideBounds( m_pos ) )
	{
		/// Impacted on the player
		theGame->m_player->AdjustHealth( -BULLETBOSS1_DAMAGE * m_damageGiveMultiplier );
		m_readyToDie = true;
	}

	/// See if it's time to die
	if( age > BULLETBOSS1_LIFESPAN_SECONDS )
	{
		m_readyToDie = true;
	}
}


//-------------------------------------------------------------------------------------------------
void Entity::Die_BulletBoss1()
{
	delete m_anim ;
}



