//-------------------------------------------------------------------------------------------------
// Entity_BulletBoss2.cpp
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#include "pch.hpp"
#include "engineAssert.hpp"
#include "Entity.hpp"
#include "Particle.hpp"
#include "TheGame.hpp"


const float BULLETBOSS2_MOVE_TILES_PER_SECOND = 8.0;
const double BULLETBOSS2_LIFESPAN_SECONDS = 4.0;
const float BULLETBOSS2_DAMAGE = 15.0f;


//-------------------------------------------------------------------------------------------------
void Entity::Spawn_BulletBoss2()
{
	m_anim = new Animation( "data/gfx/Particles/BulletBoss2.tga" );
	m_bounds.SetFromMinXYMaxXY( -8.0f, -8.0f, 8.0f, 8.0f );
	m_rotVel = 0.0f;
	m_hitPoints = 10.0f;
	m_drawRotated = true;
	m_scale.SetXY( 1.0f, 1.0f );
	m_damageGiveMultiplier = theGame->m_player->m_difficulty * 0.1f;	
	if( m_damageGiveMultiplier < 1 )
	{
		m_damageGiveMultiplier = 1;
	}
	m_damageTakeMultiplier = 0.0f;

	m_vel.SetFromCosSinDegrees( m_rot );
}


//-------------------------------------------------------------------------------------------------
void Entity::Think_BulletBoss2()
{
	double timeNow = theApplication.GetTimeFrameBegan();
	double age = timeNow - m_birthTime;

	float moveSpeed = (float)( BULLETBOSS2_MOVE_TILES_PER_SECOND * theApplication.GetFrameSeconds() );
	Vector2 moveVector = m_vel * moveSpeed;

	bool success = TryToMove( moveVector );
	if( !success )
	{
		/// Impacted on the world
		m_readyToDie = true;
	}

	/// Check if we hit the player
	if( theGame->m_player->m_worldBounds.IsPointInsideBounds( m_pos ) )
	{
		/// Impacted on the player
		theGame->m_player->AdjustHealth( -BULLETBOSS2_DAMAGE * m_damageGiveMultiplier );
		m_readyToDie = true;
	}

	/// See if it's time to die
	if( age > BULLETBOSS2_LIFESPAN_SECONDS )
	{
		m_readyToDie = true;
	}
}


//-------------------------------------------------------------------------------------------------
void Entity::Die_BulletBoss2()
{
	delete m_anim ;
}



