//-------------------------------------------------------------------------------------------------
// Entity_BulletWizrobe.cpp
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#include "pch.hpp"
#include "engineAssert.hpp"
#include "Entity.hpp"
#include "Particle.hpp"
#include "TheGame.hpp"


const float BULLETWIZROBE_MOVE_TILES_PER_SECOND = 6.0;
const double BULLETWIZROBE_LIFESPAN_SECONDS = 4.0;
const float BULLETWIZROBE_DAMAGE = 5.0f;


//-------------------------------------------------------------------------------------------------
void Entity::Spawn_BulletWizrobe()
{
	m_anim = new Animation( "data/gfx/Particles/BulletWizrobe.tga" );
	m_bounds.SetFromMinXYMaxXY( -2.0f, -2.0f, 2.0f, 2.0f );
	m_rotVel = 0.0f;
	m_hitPoints = 10.0f;
	m_drawRotated = true;
	m_scale.SetXY( 0.25f, 0.25f );
	m_damageGiveMultiplier = theGame->m_player->m_difficulty * 0.2f;	
	if( m_damageGiveMultiplier < 1 )
	{
		m_damageGiveMultiplier = 1;
	}
	m_damageTakeMultiplier = 0.0f;

	/// Calculate a normal vector towards the player
	Vector2 vectorTowardPlayer = theGame->m_player->pos - m_pos;
	if( theGame->m_player->IsInvisible() )
	{
		m_rot = RandomFloat( 0.0f, 360.0f );
	}
	else
	{
		m_rot = vectorTowardPlayer.GetDegrees();
		m_rot += RandomFloat( -15.0f, 15.0f );
	}
	m_vel.SetFromCosSinDegrees( m_rot );
}


//-------------------------------------------------------------------------------------------------
void Entity::Think_BulletWizrobe()
{
	double timeNow = theApplication.GetTimeFrameBegan();
	double age = timeNow - m_birthTime;

	float moveSpeed = (float)( BULLETWIZROBE_MOVE_TILES_PER_SECOND * theApplication.GetFrameSeconds() );
	Vector2 moveVector = m_vel * moveSpeed;

	bool success = TryToMove( moveVector );
	if( !success )
	{
		/// Impacted on the world
		m_readyToDie = true;
	}

	/// Check if we hit the player
	if( theGame->m_player->m_worldBounds.IsPointInsideBounds( m_pos ) )
	{
		/// Impacted on the player
		theGame->m_player->AdjustHealth( -BULLETWIZROBE_DAMAGE * m_damageGiveMultiplier );
		m_readyToDie = true;
	}

	/// See if it's time to die
	if( age > BULLETWIZROBE_LIFESPAN_SECONDS )
	{
		m_readyToDie = true;
	}
}


//-------------------------------------------------------------------------------------------------
void Entity::Die_BulletWizrobe()
{
	delete m_anim ;
}



