//-------------------------------------------------------------------------------------------------
// Entity_EnemyBoss1.cpp
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#include "pch.hpp"
#include "engineAssert.hpp"
#include "Entity.hpp"
#include "Particle.hpp"
#include "TheGame.hpp"


const double BOSS1_TURN_DEGREES_PER_SECOND = 80.0;
const double BOSS1_MOVE_TILES_PER_SECOND = 3.0;
const double BOSS1_LIFESPAN_SECONDS = 2600.0;
const float BOSS1_MELEE_HURT_PROXIMITY = 1.0f;
const float BOSS1_MELEE_DAMAGE_PER_SECOND = 20.0f;
const double BOSS1_WANDER_SECONDS_MIN = 0.75;
const double BOSS1_WANDER_SECONDS_MAX = 0.75;
const double BOSS1_SECONDS_TO_WAIT_BEFORE = 0.5;
const double BOSS1_SECONDS_TO_WAIT_AFTER = 0.5;
const double BOSS1_SECONDS_TO_SHOOT = 2.0;
const double BOSS1_SECONDS_TO_BEHURT = 0.1;
const float BOSS1_HURTCOLORFADE_AMMOUNT_PER_SECOND = 50;
const double BOSS1_NUMBER_OF_SHOTS_PER_ATTACK = 32.0;
const double BOSS1_SECONDS_BETWEEN_SHOTS = BOSS1_SECONDS_TO_SHOOT / BOSS1_NUMBER_OF_SHOTS_PER_ATTACK;
const char* BOSS1_ITEM1 = "Health";
const char* BOSS1_ITEM2 = "Key";

enum // wizrobe states
{
	BOSS1_STATE_WANDER,
	BOSS1_STATE_WAIT_BEFORE,
	BOSS1_STATE_SHOOTING,
	BOSS1_STATE_BOMBING,
	BOSS1_STATE_WAIT_AFTER,
};



//-------------------------------------------------------------------------------------------------
void Entity::Spawn_EnemyBoss1()
{
	theConsole.Printf( "Spawned a Boss1 at %.02f,%.02f\n", m_pos.x, m_pos.y );
	m_anim = new Animation( "data/gfx/Enemies/Boss01.tga" );
	m_bounds.SetFromMinXYMaxXY( -32.0f, -32.0f, 32.0f, 32.0f );
	m_rotVel = RandomFloat( (float)BOSS1_TURN_DEGREES_PER_SECOND, -(float)BOSS1_TURN_DEGREES_PER_SECOND );
	m_genericStateEnum = BOSS1_STATE_WANDER;
	m_hitPoints = 1000.0f * ( ( theGame->m_player->m_difficulty * theGame->m_player->m_difficulty * 0.1f ) * 0.5f );
	if( m_hitPoints < 1000.0f )
	{
		m_hitPoints = 1000.0f;
	}
	m_damageGiveMultiplier = theGame->m_player->m_difficulty * 0.25f;
	if( m_damageGiveMultiplier < 1.0f )
	{
		m_damageGiveMultiplier = 1.0f;
	}
	m_damageTakeMultiplier = theGame->m_player->m_difficulty * 0.25f;
	if( m_damageTakeMultiplier < 1.0f )
	{
		m_damageTakeMultiplier = 1.0f;
	}
	m_drawRotated = false;
	double timeNow = theApplication.GetTimeFrameBegan();
	m_genericTimeStamp = timeNow + (double) RandomFloat( (float)BOSS1_WANDER_SECONDS_MIN, (float)BOSS1_WANDER_SECONDS_MAX );
	m_isHitByPlayerBullets = false;
	m_isBeingDamagedByPlayer = false;
	m_color = Rgba( 255, 255, 255, 255 );
	m_hurtColorFade = 0;
	m_isEnemy = true;
	m_bossStage = 1;
}


//-------------------------------------------------------------------------------------------------
void Entity::Think_EnemyBoss1()
{
	double timeNow = theApplication.GetTimeFrameBegan();

	/// See if we're trampling the player
	if( theGame->IsPointWithinGivenRadiusOfPlayer( m_pos, BOSS1_MELEE_HURT_PROXIMITY ) )
	{
		theGame->m_player->AdjustHealth( ( BOSS1_MELEE_DAMAGE_PER_SECOND * m_damageGiveMultiplier ) * (float)theApplication.GetFrameSeconds() * -1.0f );
	}
	if( m_isBeingDamagedByPlayer == true )
	{
		//-------------------------------------------------------------------------------------------------
		// BEING HURT
		//-------------------------------------------------------------------------------------------------
		m_hurtColorFade = m_hurtColorFade + BOSS1_HURTCOLORFADE_AMMOUNT_PER_SECOND;
		m_color = Rgba( 255, m_hurtColorFade, m_hurtColorFade, 255 );

		/// See if it's time to stop appearing yet
		if( m_color.b > 200 )
		{
			m_color = Rgba( 255, 255, 255, 255 );
			m_hurtColorFade = 0;
			m_isBeingDamagedByPlayer = false;
		}
	}

	if( m_genericStateEnum == BOSS1_STATE_WANDER )
	{
		//-------------------------------------------------------------------------------------------------
		// WANDER
		//-------------------------------------------------------------------------------------------------
		m_isHitByPlayerBullets = true;

		/// See if it's time to stop wandering yet
		if( timeNow > m_genericTimeStamp )
		{
			float Boss1_BombTime = RandomFloat( 0.0f, 1.0f );
			if( Boss1_BombTime > 0.3 )
			{
				m_genericStateEnum = BOSS1_STATE_BOMBING;
				m_genericTimeStamp = timeNow + BOSS1_SECONDS_TO_WAIT_BEFORE;	
			}
			else
			{
				m_genericStateEnum = BOSS1_STATE_WAIT_BEFORE;
				m_genericTimeStamp = timeNow + BOSS1_SECONDS_TO_WAIT_BEFORE;
			}
		}

		/// Wander
		const double turnSpeed = m_rotVel * theApplication.GetFrameSeconds();
		const double moveSpeed = BOSS1_MOVE_TILES_PER_SECOND * theApplication.GetFrameSeconds();

		/// Build move vector
		Vector2 moveVector;

		Vector2 vecToPlayer = theGame->m_player->pos - m_pos;
		float degreesToPlayer;
		degreesToPlayer = (float) vecToPlayer.GetDegrees();
		m_rot = degreesToPlayer;

		moveVector.SetFromRadiusCosSinDegrees( moveSpeed, m_rot );
		bool success = TryToMove( moveVector );
	}
	else if( m_genericStateEnum == BOSS1_STATE_WAIT_BEFORE )
	{
		//-------------------------------------------------------------------------------------------------
		// WAIT BEFORE AN ACTION
		//-------------------------------------------------------------------------------------------------
		/// Check if it is time to move onto the next start
		if( timeNow > m_genericTimeStamp )
		{
			m_genericStateEnum = BOSS1_STATE_SHOOTING;
			m_genericTimeStamp = timeNow + BOSS1_SECONDS_TO_SHOOT;
			m_genericTimeStamp2 = timeNow;
		}
	}
	else if( m_genericStateEnum == BOSS1_STATE_SHOOTING )
	{
		//-------------------------------------------------------------------------------------------------
		// SHOOTING
		//-------------------------------------------------------------------------------------------------
		/// Check if it's time to stop shooting
		if( timeNow > m_genericTimeStamp )
		{
			m_genericStateEnum = BOSS1_STATE_WAIT_AFTER;
			m_genericTimeStamp = timeNow + BOSS1_SECONDS_TO_WAIT_AFTER;
		}

		/// Check if it's time to shoot again yet
		if( timeNow > m_genericTimeStamp2 )
		{
			m_genericTimeStamp2 = timeNow + BOSS1_SECONDS_BETWEEN_SHOTS; // next shoot time
			Boss1Shoot1();
		}
	}
	else if( m_genericStateEnum == BOSS1_STATE_BOMBING )
	{
		//-------------------------------------------------------------------------------------------------
		// BOMBING
		//-------------------------------------------------------------------------------------------------
		Boss1Shoot2();
		m_genericStateEnum = BOSS1_STATE_WANDER;
		m_genericTimeStamp = timeNow + (double) RandomFloat( (float)BOSS1_WANDER_SECONDS_MIN, (float)BOSS1_WANDER_SECONDS_MAX );
	}
	else if( m_genericStateEnum == BOSS1_STATE_WAIT_AFTER )
	{
		//-------------------------------------------------------------------------------------------------
		// WAIT AFTER AN ACTION
		//-------------------------------------------------------------------------------------------------
		/// Check if it is time to move onto the next state
		if( timeNow > m_genericTimeStamp )
		{
			m_genericStateEnum = BOSS1_STATE_WANDER;
			m_genericTimeStamp = timeNow + (double) RandomFloat( (float)BOSS1_WANDER_SECONDS_MIN, (float)BOSS1_WANDER_SECONDS_MAX );
		}
	}
	else
	{
		m_genericStateEnum = BOSS1_STATE_WANDER;
	}

	/// See if it's time to die
	double age = timeNow - m_birthTime;
	if( age > BOSS1_LIFESPAN_SECONDS )
	{
		m_readyToDie = true;
	}
}


//-------------------------------------------------------------------------------------------------
void Entity::Die_EnemyBoss1()
{
	SpawnNewEntity( ENT_ITEMPICKUP_KEY, m_pos );
	float randomSpawnPoint = RandomFloat( -2 , 2 );
	SpawnNewEntity( ENT_ITEMPICKUP_HEALTH, m_pos + randomSpawnPoint );
	randomSpawnPoint = RandomFloat( -2 , 2 );
	SpawnNewEntity( ENT_ITEMPICKUP_HEALTH, m_pos + randomSpawnPoint );
	randomSpawnPoint = RandomFloat( -2 , 2 );
	SpawnNewEntity( ENT_ITEMPICKUP_HEALTH, m_pos + randomSpawnPoint );

	delete m_anim ;

	Particle* p = Particle::Spawn();
	p->m_pos = m_pos;
	p->m_rotVel = RandomFloat( 400.0f, 800.0f );
	if( RandomInt(2) )
	{
		p->m_rotVel *= -1.0f;
	}
	p->m_startAlpha = 1.0f;
	p->m_endAlpha = 0.0f;
	p->m_startScale = 1.0f;
	p->m_endScale = 5.0f;
	p->m_timeToLive = 0.4;
	p->m_anim = new Animation( "data/gfx/Enemies/Boss01.tga" );
}



//-------------------------------------------------------------------------------------------------
// Boss1Shoot1
//-------------------------------------------------------------------------------------------------
void Entity::Boss1Shoot1()
{
	double timeNow = theApplication.GetTimeFrameBegan();

	/// Spawn a new wizrobe bullet aimed at the player
	Entity& ent = SpawnNewEntity( ENT_BULLET_BOSS1, m_pos );
	m_timeOfLastSpawn = timeNow;
}

//-------------------------------------------------------------------------------------------------
// Boss1Shoot2
//-------------------------------------------------------------------------------------------------
void Entity::Boss1Shoot2()
{
	double timeNow = theApplication.GetTimeFrameBegan();

	/// Spawn a new wizrobe bullet aimed at the player
	Entity& ent = SpawnNewEntity( ENT_BOMB, m_pos );
	m_timeOfLastSpawn = timeNow;
}

