//-------------------------------------------------------------------------------------------------
// Entity_EnemyBoss2.cpp
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#include "pch.hpp"
#include "engineAssert.hpp"
#include "Entity.hpp"
#include "Particle.hpp"
#include "TheGame.hpp"


const double BOSS2_TURN_DEGREES_PER_SECOND = 45.0;
const double BOSS2_MOVE_TILES_PER_SECOND = 4.0;
const double BOSS2_LIFESPAN_SECONDS = 2600.0;
const float BOSS2_MELEE_HURT_PROXIMITY = 1.0f;
const float BOSS2_MELEE_DAMAGE_PER_SECOND = 20.0f;
const double BOSS2_WANDER_SECONDS_MIN = 2.0;
const double BOSS2_WANDER_SECONDS_MAX = 2.0;
const double BOSS2_SECONDS_TO_WAIT_BEFORE = 0.0;
const double BOSS2_SECONDS_TO_WAIT_AFTER = 0.0;
const double BOSS2_SECONDS_TO_SHOOT = 0.0;
const double BOSS2_SECONDS_TO_BEHURT = 0.1;
const float BOSS2_HURTCOLORFADE_AMMOUNT_PER_SECOND = 50;
const double BOSS2_NUMBER_OF_SHOTS_PER_ATTACK = 32.0;
const double BOSS2_SECONDS_BETWEEN_SHOTS = BOSS2_SECONDS_TO_SHOOT / BOSS2_NUMBER_OF_SHOTS_PER_ATTACK;
const char* BOSS2_ITEM1 = "Health";
const char* BOSS2_ITEM2 = "Key";
const float BOSS2_STAGE2_CHANGE_AT_HITPOINT_VALUE = 800.0f;
const float BOSS2_STAGE3_CHANGE_AT_HITPOINT_VALUE = 400.0f;

enum // wizrobe states
{
	BOSS2_STATE_WANDER,
	BOSS2_STATE_WAIT_BEFORE,
	BOSS2_STATE_SHOOTING,
	BOSS2_STATE_WAIT_AFTER,
};



//-------------------------------------------------------------------------------------------------
void Entity::Spawn_EnemyBoss2()
{
	theConsole.Printf( "Spawned a Boss2 at %.02f,%.02f\n", m_pos.x, m_pos.y );
	m_anim = new Animation( "data/gfx/Enemies/Boss02.tga" );
	m_bounds.SetFromMinXYMaxXY( -32.0f, -32.0f, 32.0f, 32.0f );
	m_rotVel = BOSS2_TURN_DEGREES_PER_SECOND;
	m_genericStateEnum = BOSS2_STATE_WANDER;
	m_hitPoints = 1000.0f * ( ( theGame->m_player->m_difficulty * theGame->m_player->m_difficulty * 0.1 ) * 0.5 );
	if( m_hitPoints < 1000 )
	{
		m_hitPoints = 1000;
	}
	m_damageGiveMultiplier = theGame->m_player->m_difficulty * 0.25;
	if( m_damageGiveMultiplier < 1 )
	{
		m_damageGiveMultiplier = 1;
	}
	m_damageTakeMultiplier = theGame->m_player->m_difficulty * 0.25;
	if( m_damageTakeMultiplier < 1 )
	{
		m_damageTakeMultiplier = 1;
	}
	m_drawRotated = false;
	double timeNow = theApplication.GetTimeFrameBegan();
	m_genericTimeStamp = timeNow + (double) RandomFloat( (float)BOSS2_WANDER_SECONDS_MIN, (float)BOSS2_WANDER_SECONDS_MAX );
	m_isHitByPlayerBullets = false;
	m_isBeingDamagedByPlayer = false;
	m_color = Rgba( 255, 255, 255, 255 );
	m_hurtColorFade = 0;
	m_isEnemy = true;
	m_bossStage = 1;
}


//-------------------------------------------------------------------------------------------------
void Entity::Think_EnemyBoss2()
{
	double timeNow = theApplication.GetTimeFrameBegan();

	/// See if we're trampling the player
	if( theGame->IsPointWithinGivenRadiusOfPlayer( m_pos, BOSS2_MELEE_HURT_PROXIMITY ) )
	{
		theGame->m_player->AdjustHealth( ( BOSS2_MELEE_DAMAGE_PER_SECOND * m_damageGiveMultiplier ) * (float)theApplication.GetFrameSeconds() * -1.0f );
	}
	float boss2Stage2ChangeAtHitpointValue = BOSS2_STAGE2_CHANGE_AT_HITPOINT_VALUE * ( theGame->m_player->m_difficulty * 0.25 );
	if( boss2Stage2ChangeAtHitpointValue < BOSS2_STAGE2_CHANGE_AT_HITPOINT_VALUE )
	{
		boss2Stage2ChangeAtHitpointValue = BOSS2_STAGE2_CHANGE_AT_HITPOINT_VALUE;
	}
	float boss2Stage3ChangeAtHitpointValue = BOSS2_STAGE3_CHANGE_AT_HITPOINT_VALUE * ( theGame->m_player->m_difficulty * 0.25 );
	if( boss2Stage3ChangeAtHitpointValue < BOSS2_STAGE3_CHANGE_AT_HITPOINT_VALUE )
	{
		boss2Stage3ChangeAtHitpointValue = BOSS2_STAGE3_CHANGE_AT_HITPOINT_VALUE;
	}
	if( m_bossStage == 1 && m_hitPoints < boss2Stage2ChangeAtHitpointValue )
	{
		m_bossStage = 2;
	}
	else if( m_bossStage == 2 && m_hitPoints < boss2Stage3ChangeAtHitpointValue )
	{
		m_bossStage = 3;
	}
	if( m_isBeingDamagedByPlayer == true )
	{
		//-------------------------------------------------------------------------------------------------
		// BEING HURT
		//-------------------------------------------------------------------------------------------------
		m_hurtColorFade = m_hurtColorFade + BOSS2_HURTCOLORFADE_AMMOUNT_PER_SECOND;
		m_color = Rgba( 255, m_hurtColorFade, m_hurtColorFade, 255 );

		/// See if it's time to stop appearing yet
		if( m_color.b > 200 )
		{
			m_color = Rgba( 255, 255, 255, 255 );
			m_hurtColorFade = 0;
			m_isBeingDamagedByPlayer = false;
			
			if( m_bossStage == 1 )
			{
				SpawnNewEntity( ENT_ENEMY_MUSHROOM, m_pos );
			}
			else
			{
				SpawnNewEntity( ENT_ENEMY_MUSHROOM, m_pos );
				SpawnNewEntity( ENT_ENEMY_MUSHROOM, m_pos );
			}

			/// Turn towards the player
			Vector2 vecToPlayer = theGame->m_player->pos - m_pos;
			m_rot = (float) vecToPlayer.GetDegrees();
		}
	}

	if( m_genericStateEnum == BOSS2_STATE_WANDER )
	{
		//-------------------------------------------------------------------------------------------------
		// WANDER
		//-------------------------------------------------------------------------------------------------
		m_isHitByPlayerBullets = true;

		/// See if it's time to stop wandering yet
		if( timeNow > m_genericTimeStamp )
		{
			m_genericStateEnum = BOSS2_STATE_WAIT_BEFORE;
			m_genericTimeStamp = timeNow + BOSS2_SECONDS_TO_WAIT_BEFORE;
		}
		
		/// Wander
		const double turnSpeed = m_rotVel * theApplication.GetFrameSeconds();
		double moveSpeed = BOSS2_MOVE_TILES_PER_SECOND * theApplication.GetFrameSeconds();
		if( m_bossStage == 2 )
		{
			moveSpeed = ( BOSS2_MOVE_TILES_PER_SECOND + 1 ) * theApplication.GetFrameSeconds();
		}
		else if( m_bossStage == 3 )
		{
			moveSpeed = ( BOSS2_MOVE_TILES_PER_SECOND + 2 ) * theApplication.GetFrameSeconds();
		}

		/// Constant rotation
		m_rot -= (float) turnSpeed;
		m_rot = fmod( m_rot, 360.0 );

		/// Build move vector
		Vector2 moveVector;
		moveVector.SetFromRadiusCosSinDegrees( moveSpeed, m_rot );
		bool success = TryToMove( moveVector );
		if( !success )
		{
			m_rot -= 90.0f;
		}
	}
	else if( m_genericStateEnum == BOSS2_STATE_WAIT_BEFORE )
	{
		//-------------------------------------------------------------------------------------------------
		// WAIT BEFORE AN ACTION
		//-------------------------------------------------------------------------------------------------
		/// Check if it is time to move onto the next start
		if( timeNow > m_genericTimeStamp )
		{
			m_genericStateEnum = BOSS2_STATE_SHOOTING;
			m_genericTimeStamp = timeNow + BOSS2_SECONDS_TO_SHOOT;
			m_genericTimeStamp2 = timeNow;
		}
	}
	else if( m_genericStateEnum == BOSS2_STATE_SHOOTING )
	{
		//-------------------------------------------------------------------------------------------------
		// SHOOTING
		//-------------------------------------------------------------------------------------------------
		/// Check if it's time to stop shooting
		if( timeNow > m_genericTimeStamp )
		{
			m_genericStateEnum = BOSS2_STATE_WAIT_AFTER;
			m_genericTimeStamp = timeNow + BOSS2_SECONDS_TO_WAIT_AFTER;
		}

		/// Check if it's time to shoot again yet
		if( timeNow > m_genericTimeStamp2 && m_bossStage == 2 )
		{
			m_genericTimeStamp2 = timeNow + BOSS2_SECONDS_BETWEEN_SHOTS; // next shoot time

			/// Spawn 4 Boss2 bullets
			timeNow = theApplication.GetTimeFrameBegan();
			SpawnNewEntity( ENT_BULLET_BOSS2, m_pos , 45 );
			SpawnNewEntity( ENT_BULLET_BOSS2, m_pos , 135 );
			SpawnNewEntity( ENT_BULLET_BOSS2, m_pos , 225 );
			SpawnNewEntity( ENT_BULLET_BOSS2, m_pos , 315 );
			m_timeOfLastSpawn = timeNow;
		}
		/// Check if it's time to shoot again yet
		else if( timeNow > m_genericTimeStamp2 && m_bossStage == 3 )
		{
			m_genericTimeStamp2 = timeNow + BOSS2_SECONDS_BETWEEN_SHOTS; // next shoot time

			/// Spawn 8 Boss2 bullets
			timeNow = theApplication.GetTimeFrameBegan();
			SpawnNewEntity( ENT_BULLET_BOSS2, m_pos , 0 );
			SpawnNewEntity( ENT_BULLET_BOSS2, m_pos , 45 );
			SpawnNewEntity( ENT_BULLET_BOSS2, m_pos , 90 );
			SpawnNewEntity( ENT_BULLET_BOSS2, m_pos , 135 );
			SpawnNewEntity( ENT_BULLET_BOSS2, m_pos , 180 );
			SpawnNewEntity( ENT_BULLET_BOSS2, m_pos , 225 );
			SpawnNewEntity( ENT_BULLET_BOSS2, m_pos , 270 );
			SpawnNewEntity( ENT_BULLET_BOSS2, m_pos , 315 );
			m_timeOfLastSpawn = timeNow;
		}
	}
	else if( m_genericStateEnum == BOSS2_STATE_WAIT_AFTER )
	{
		//-------------------------------------------------------------------------------------------------
		// WAIT AFTER AN ACTION
		//-------------------------------------------------------------------------------------------------
		/// Check if it is time to move onto the next state
		if( timeNow > m_genericTimeStamp )
		{
			m_genericStateEnum = BOSS2_STATE_WANDER;
			m_genericTimeStamp = timeNow + (double) RandomFloat( (float)BOSS2_WANDER_SECONDS_MIN, (float)BOSS2_WANDER_SECONDS_MAX );
		}
	}
	else
	{
		m_genericStateEnum = BOSS2_STATE_WANDER;
	}

	/// See if it's time to die
	double age = timeNow - m_birthTime;
	if( age > BOSS2_LIFESPAN_SECONDS )
	{
		m_readyToDie = true;
	}
}


//-------------------------------------------------------------------------------------------------
void Entity::Die_EnemyBoss2()
{
	DamageEntitiesRadius( m_pos, 14, 100 * m_damageGiveMultiplier );

	SpawnNewEntity( ENT_ITEMPICKUP_KEY, m_pos );
	float randomSpawnPoint = RandomFloat( -2 , 2 );
	SpawnNewEntity( ENT_ITEMPICKUP_HEALTH, m_pos + randomSpawnPoint );
	randomSpawnPoint = RandomFloat( -2 , 2 );
	SpawnNewEntity( ENT_ITEMPICKUP_HEALTH, m_pos + randomSpawnPoint );
	randomSpawnPoint = RandomFloat( -2 , 2 );
	SpawnNewEntity( ENT_ITEMPICKUP_HEALTH, m_pos + randomSpawnPoint );
	
	delete m_anim ;

	Particle* p = Particle::Spawn();
	p->m_pos = m_pos;
	p->m_rotVel = RandomFloat( 400.0f, 800.0f );
	if( RandomInt(2) )
	{
		p->m_rotVel *= -1.0f;
	}
	p->m_startAlpha = 1.0f;
	p->m_endAlpha = 0.0f;
	p->m_startScale = 1.0f;
	p->m_endScale = 5.0f;
	p->m_timeToLive = 0.4;
	p->m_anim = new Animation( "data/gfx/Enemies/Boss02.tga" );
}



//-------------------------------------------------------------------------------------------------
// Boss2Shoot1
//-------------------------------------------------------------------------------------------------
void Entity::Boss2Shoot1()
{
	double timeNow = theApplication.GetTimeFrameBegan();

	/// Spawn a new wizrobe bullet aimed at the player
	SpawnNewEntity( ENT_BULLET_BOSS2, m_pos , 45 );
	SpawnNewEntity( ENT_BULLET_BOSS2, m_pos , 135 );
	SpawnNewEntity( ENT_BULLET_BOSS2, m_pos , 225 );
	SpawnNewEntity( ENT_BULLET_BOSS2, m_pos , 315 );
	m_timeOfLastSpawn = timeNow;
}


