//-------------------------------------------------------------------------------------------------
// Entity_EnemyMushroom.cpp
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#include "pch.hpp"
#include "engineAssert.hpp"
#include "Entity.hpp"
#include "Particle.hpp"
#include "TheGame.hpp"


const double MUSHROOM_TURN_DEGREES_PER_SECOND = 30.0;
const double MUSHROOM_MOVE_TILES_PER_SECOND = 2.0;
const double MUSHROOM_MOVE_TILES_PER_SECOND_WHILE_STUNNED = 1.5;
const double MUSHROOM_CHARGE_TILES_PER_SECOND = 5.0;
const double MUSHROOM_LIFESPAN_SECONDS = 30.0;
const float MUSHROOM_HURTCOLORFADE_AMMOUNT_PER_SECOND = 50;
const float MUSHROOM_EXPLODE_PLAYER_PROXIMITY = 3.0f;
const double MUSHROOM_WANDER_SECONDS_MIN = 1.0;
const double MUSHROOM_WANDER_SECONDS_MAX = 2.0;
const double MUSHROOM_SECONDS_WAIT_BEFORE = 1.0;
const double MUSHROOM_SECONDS_WAIT_AFTER = 0.1;
const double MUSHROOM_SECONDS_TO_SHOOT = 1.0;
const double MUSHROOM_NUMBER_OF_SHOTS_PER_APPEARANCE = 1.0;
const double MUSHROOM_SECONDS_BETWEEN_SHOTS = MUSHROOM_SECONDS_TO_SHOOT / MUSHROOM_NUMBER_OF_SHOTS_PER_APPEARANCE;
const int MUSHROOM_WANDER_ALPHA = 0;
const float MUSHROOM_STUN_TIME = 4.0f;
const float MUSHROOM_EXPLOSION_VISUAL_RADIUS = 3.0f;
const float MUSHROOM_EXPLOSION_DAMAGE_RADIUS = 3.0f;
const float MUSHROOM_EXPLOSION_MAX_DAMAGE = 5.0f;

enum // grunt states
{
	MUSHROOM_STATE_WANDER,
	MUSHROOM_STATE_WAIT_BEFORE,
	MUSHROOM_STATE_SHOOTING,
	MUSHROOM_STATE_WAIT_AFTER,
	MUSHROOM_STATE_STUNNED,
};


//-------------------------------------------------------------------------------------------------
// Spawn_EnemyMushroom
//-------------------------------------------------------------------------------------------------
void Entity::Spawn_EnemyMushroom()
{
	m_anim = new Animation( "data/gfx/Enemies/Mushroom.tga" );
	m_bounds.SetFromMinXYMaxXY( -8.0f, -8.0f, 8.0f, 8.0f );
	m_rotVel = RandomFloat( (float)MUSHROOM_TURN_DEGREES_PER_SECOND, -(float)MUSHROOM_TURN_DEGREES_PER_SECOND );
	m_rot = RandomFloat( 0 , 359 );
	m_genericStateEnum = MUSHROOM_STATE_WANDER;
	m_hitPoints = 10.0f * ( ( theGame->m_player->m_difficulty * theGame->m_player->m_difficulty * 0.1 ) * 0.5 );
	if( m_hitPoints < 10 )
	{
		m_hitPoints = 10;
	}
	m_damageGiveMultiplier = theGame->m_player->m_difficulty * 0.25;
	if( m_damageGiveMultiplier < 1 )
	{
		m_damageGiveMultiplier = 1;
	}
	m_damageTakeMultiplier = theGame->m_player->m_difficulty * 0.25;
	if( m_damageTakeMultiplier < 1 )
	{
		m_damageTakeMultiplier = 1;
	}
	m_drawRotated = false;
	m_isHitByPlayerBullets = true;
	m_hurtColorFade = 100;
	m_isEnemy = true;
	double timeNow = theApplication.GetTimeFrameBegan();
}


//-------------------------------------------------------------------------------------------------
// Think_EnemyMushroom
//-------------------------------------------------------------------------------------------------
void Entity::Think_EnemyMushroom()
{
	double timeNow = theApplication.GetTimeFrameBegan();
	
	if( m_stunned == true && m_genericStateEnum != MUSHROOM_STATE_STUNNED )
	{
		m_genericStateEnum = MUSHROOM_STATE_STUNNED;
		m_genericTimeStamp = timeNow + MUSHROOM_STUN_TIME;
	}

	if( m_genericStateEnum == MUSHROOM_STATE_WANDER )
	{
		//-------------------------------------------------------------------------------------------------
		// WANDER
		//-------------------------------------------------------------------------------------------------
		m_color = Rgba( 255, 255, 255, 255 );

		/// Explode if the player is nearby
		if( !theGame->m_player->IsInvisible() && theGame->IsPointWithinGivenRadiusOfPlayer( m_pos, MUSHROOM_EXPLODE_PLAYER_PROXIMITY ) )
		{
			/// Charge the player if he is nearby
			m_genericStateEnum = MUSHROOM_STATE_WAIT_BEFORE;
			m_genericTimeStamp = timeNow + MUSHROOM_SECONDS_WAIT_BEFORE;
		}
		else
		{
			/// Wander
			const double turnSpeed = m_rotVel * theApplication.GetFrameSeconds();
			const double moveSpeed = MUSHROOM_MOVE_TILES_PER_SECOND * theApplication.GetFrameSeconds();

			/// Constant rotation
			m_rot += (float) turnSpeed;
			m_rot = fmod( m_rot, 360.0 );

			/// Build move vector
			Vector2 moveVector;
			moveVector.SetFromRadiusCosSinDegrees( moveSpeed, m_rot );
			bool success = TryToMove( moveVector );
			if( !success )
			{
				m_rot += 30.0f;
			}
		}
	}
	else if( m_genericStateEnum == MUSHROOM_STATE_WAIT_BEFORE )
	{
		//-------------------------------------------------------------------------------------------------
		// WAIT BEFORE AN ACTION
		//-------------------------------------------------------------------------------------------------
		/// Check if it is time to move onto the next start
		if( timeNow > m_genericTimeStamp )
		{
			m_genericStateEnum = MUSHROOM_STATE_SHOOTING;
			m_genericTimeStamp = timeNow + MUSHROOM_SECONDS_TO_SHOOT;
			m_genericTimeStamp2 = timeNow;
		}

		m_hurtColorFade = m_hurtColorFade + MUSHROOM_HURTCOLORFADE_AMMOUNT_PER_SECOND;
		m_color = Rgba( m_hurtColorFade, 255, m_hurtColorFade, 255 );

		/// See if it's time to stop appearing yet
		if( m_color.b > 200 )
		{
			m_color = Rgba( 255, 255, 255, 255 );
			m_hurtColorFade = 100;
			m_isBeingDamagedByPlayer = false;
		}
	}
	else if( m_genericStateEnum == MUSHROOM_STATE_SHOOTING )
	{
		//-------------------------------------------------------------------------------------------------
		// SHOOTING
		//-------------------------------------------------------------------------------------------------
		m_color = Rgba( 255, 255, 255, 255 );

		m_isHitByPlayerBullets = true;

		/// See if it's time to stop appearing yet
		if( timeNow > m_genericTimeStamp )
		{
			m_genericStateEnum = MUSHROOM_STATE_WAIT_AFTER;
			m_genericTimeStamp = timeNow + MUSHROOM_SECONDS_WAIT_AFTER;
		}

		/// See if it's time to shoot again yet
		if( timeNow > m_genericTimeStamp2 )
		{
			m_genericTimeStamp2 = timeNow + MUSHROOM_SECONDS_BETWEEN_SHOTS; // next shoot time
			MushroomShoot();
		}
	}
	else if( m_genericStateEnum == MUSHROOM_STATE_WAIT_AFTER )
	{
		//-------------------------------------------------------------------------------------------------
		// WAIT AFTER AN ACTION
		//-------------------------------------------------------------------------------------------------
		/// Check if it is time to move onto the next state
		if( timeNow > m_genericTimeStamp )
		{
			m_genericStateEnum = MUSHROOM_STATE_WANDER;
			m_genericTimeStamp = timeNow + (double) RandomFloat( (float)MUSHROOM_WANDER_SECONDS_MIN, (float)MUSHROOM_WANDER_SECONDS_MAX );
		}
	}
	else if( m_genericStateEnum == MUSHROOM_STATE_STUNNED )
	{
		//-------------------------------------------------------------------------------------------------
		// STUNNED
		//-------------------------------------------------------------------------------------------------

		/// Wander around stunned
		const double turnSpeed = m_rotVel * theApplication.GetFrameSeconds();
		const double moveSpeed = MUSHROOM_MOVE_TILES_PER_SECOND_WHILE_STUNNED * theApplication.GetFrameSeconds();

		m_color = Rgba( 200, 255, 0, 255 );

		/// Constant rotation
		m_rot += (float) turnSpeed;
		m_rot = fmod( m_rot, 360.0 );

		/// Build move vector
		Vector2 moveVector;
		moveVector.SetFromRadiusCosSinDegrees( moveSpeed, m_rot );
		bool success = TryToMove( moveVector );
		if( !success )
		{
			m_rot += 30.0f;
		}

		/// See if it's time to become unstunned
		if( timeNow > m_genericTimeStamp )
		{
			m_genericStateEnum = MUSHROOM_STATE_WAIT_AFTER;
			m_genericTimeStamp = timeNow + MUSHROOM_SECONDS_WAIT_AFTER;
			m_stunned = false;
		}
	}
	else
	{
		m_genericStateEnum = MUSHROOM_STATE_WANDER;
	}

	/// See if it's time to die
	double age = timeNow - m_birthTime;
	if( age > MUSHROOM_LIFESPAN_SECONDS )
	{
		m_readyToDie = true;
	}
}


//-------------------------------------------------------------------------------------------------
// Die_EnemyMushroom
//-------------------------------------------------------------------------------------------------
void Entity::Die_EnemyMushroom()
{

	int randomItemSpawn = RandomInt( 0 , 19 );
	if( randomItemSpawn == 1 )
	{
		SpawnNewEntity( ENT_ITEMPICKUP_STUNNER, m_pos );
	}

	delete m_anim ;

	Particle* p = Particle::Spawn();
	p->m_pos = m_pos;
	p->m_rotVel = RandomFloat( 400.0f, 800.0f );
	if( RandomInt(2) )
	{
		p->m_rotVel *= -1.0f;
	}
	p->m_startAlpha = 1.0f;
	p->m_endAlpha = 0.0f;
	p->m_startScale = 1.0f;
	p->m_endScale = 5.0f;
	p->m_timeToLive = 0.3;
	p->m_anim = new Animation( "data/gfx/enemies/Mushroom.tga" );
}


//-------------------------------------------------------------------------------------------------
// MushroomShoot
//-------------------------------------------------------------------------------------------------
void Entity::MushroomShoot()
{
	Particle* p = Particle::Spawn();
	p->m_pos = m_pos;
	p->m_startAlpha = 1.0f;
	p->m_endAlpha = 0.0f;
	p->m_startScale = 0.0f;
	p->m_endScale = MUSHROOM_EXPLOSION_VISUAL_RADIUS;
	p->m_timeToLive = 0.35;
	p->m_anim = new Animation( "data/gfx/particles/MushroomExplode.tga" );

	/// Check if we hit the player
	if( theGame->IsPointWithinGivenRadiusOfPlayer( m_pos, MUSHROOM_EXPLOSION_DAMAGE_RADIUS ) )
	{
		theGame->m_player->AdjustHealth( -MUSHROOM_EXPLOSION_MAX_DAMAGE * m_damageGiveMultiplier );
	}
}

