//-------------------------------------------------------------------------------------------------
// Entity_EnemyWizrobe.cpp
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#include "pch.hpp"
#include "engineAssert.hpp"
#include "Entity.hpp"
#include "Particle.hpp"
#include "TheGame.hpp"


const double WIZROBE_TURN_DEGREES_PER_SECOND = 80.0;
const double WIZROBE_MOVE_TILES_PER_SECOND = 20.0;
const double WIZROBE_MOVE_TILES_PER_SECOND_WHILE_STUNNED = 1.5;
const double WIZROBE_CHARGE_TILES_PER_SECOND = 5.0;
const double WIZROBE_LIFESPAN_SECONDS = 30.0;
const float WIZROBE_CHARGE_PLAYER_PROXIMITY = 3.0f;
const float WIZROBE_CHARGE_HURT_PROXIMITY = 0.5f;
const double WIZROBE_CHARGE_WAIT_SECONDS = 0.0;
const double WIZROBE_PAUSE_AFTER_CHARGE_SECONDS = 0.0;
const double WIZROBE_WANDER_SECONDS_MIN = 1.0;
const double WIZROBE_WANDER_SECONDS_MAX = 2.0;
const double WIZROBE_SECONDS_TO_APPEAR = 0.5;
const double WIZROBE_SECONDS_TO_DISAPPEAR = 0.5;
const double WIZROBE_SECONDS_TO_SHOOT = 2.0;
const double WIZROBE_NUMBER_OF_SHOTS_PER_APPEARANCE = 3.0;
const double WIZROBE_SECONDS_BETWEEN_SHOTS = WIZROBE_SECONDS_TO_SHOOT / WIZROBE_NUMBER_OF_SHOTS_PER_APPEARANCE;
const int WIZROBE_WANDER_ALPHA = 0;
const float WIZROBE_STUN_TIME = 4.0f;


enum // wizrobe states
{
	WIZROBE_STATE_WANDER,
	WIZROBE_STATE_APPEARING,
	WIZROBE_STATE_SHOOTING,
	WIZROBE_STATE_DISAPPEARING,
	WIZROBE_STATE_STUNNED,
};



//-------------------------------------------------------------------------------------------------
void Entity::Spawn_EnemyWizrobe()
{
	theConsole.Printf( "Spawned a Wizrobe at %.02f,%.02f\n", m_pos.x, m_pos.y );
	m_anim = new Animation( "data/gfx/Enemies/Wizard01.tga" );
	m_bounds.SetFromMinXYMaxXY( -8.0f, -8.0f, 8.0f, 8.0f );
	m_rotVel = RandomFloat( (float)WIZROBE_TURN_DEGREES_PER_SECOND, -(float)WIZROBE_TURN_DEGREES_PER_SECOND );
	m_genericStateEnum = WIZROBE_STATE_WANDER;
	m_hitPoints = 10.0f * ( ( theGame->m_player->m_difficulty * theGame->m_player->m_difficulty * 0.1 ) * 0.5 );
	if( m_hitPoints < 10 )
	{
		m_hitPoints = 10;
	}
	m_damageTakeMultiplier = theGame->m_player->m_difficulty * 0.25;
	if( m_damageTakeMultiplier < 1 )
	{
		m_damageTakeMultiplier = 1;
	}
	m_drawRotated = false;
	double timeNow = theApplication.GetTimeFrameBegan();
	m_genericTimeStamp = timeNow + (double) RandomFloat( (float)WIZROBE_WANDER_SECONDS_MIN, (float)WIZROBE_WANDER_SECONDS_MAX );
	m_isHitByPlayerBullets = false;
	m_isEnemy = true;
}


//-------------------------------------------------------------------------------------------------
void Entity::Think_EnemyWizrobe()
{
	double timeNow = theApplication.GetTimeFrameBegan();

	if( m_color.a != 255 )
	{
		m_stunned = false;
	}

	if( m_stunned == true && m_genericStateEnum != WIZROBE_STATE_STUNNED )
	{
		m_genericStateEnum = WIZROBE_STATE_STUNNED;
		m_genericTimeStamp = timeNow + WIZROBE_STUN_TIME;
	}

	if( m_genericStateEnum == WIZROBE_STATE_WANDER )
	{
		//-------------------------------------------------------------------------------------------------
		// WANDER
		//-------------------------------------------------------------------------------------------------
		m_isHitByPlayerBullets = false;

		/// See if it's time to stop wandering yet
		if( timeNow > m_genericTimeStamp )
		{
			m_genericStateEnum = WIZROBE_STATE_APPEARING;
			m_genericTimeStamp = timeNow + WIZROBE_SECONDS_TO_APPEAR;
		}

		m_color = Rgba( 255, 255, 255, WIZROBE_WANDER_ALPHA );

		/// Wander
		const double turnSpeed = m_rotVel * theApplication.GetFrameSeconds();
		const double moveSpeed = WIZROBE_MOVE_TILES_PER_SECOND * theApplication.GetFrameSeconds();

		/// Constant rotation
		m_rot += (float) turnSpeed;
		m_rot = fmod( m_rot, 360.0 );

		/// Build move vector
		Vector2 moveVector;
		moveVector.SetFromRadiusCosSinDegrees( moveSpeed, m_rot );
		bool success = TryToMove( moveVector );
		if( !success )
		{
			Vector2 vecToPlayer = theGame->m_player->pos - m_pos;
			float degreesToPlayer;
			if( theGame->m_player->IsInvisible() )
			{
				degreesToPlayer = RandomFloat( 0.0f, 360.0f );
			}
			else
			{
				degreesToPlayer = (float) vecToPlayer.GetDegrees();
				degreesToPlayer += RandomFloat( -90.0f, 90.0f );
			}
			m_rot = degreesToPlayer;
		}
	}
	else if( m_genericStateEnum == WIZROBE_STATE_APPEARING )
	{
		//-------------------------------------------------------------------------------------------------
		// APPEARING
		//-------------------------------------------------------------------------------------------------
		double timeStartAppearing = m_genericTimeStamp - WIZROBE_SECONDS_TO_APPEAR;
		double timeFinishAppearing = m_genericTimeStamp;
		double fractionDoneAppearing = (timeNow - timeStartAppearing) / (timeFinishAppearing - timeStartAppearing);
		fractionDoneAppearing = ClampDouble( fractionDoneAppearing, 0.0f, 1.0f );
		int alpha = WIZROBE_WANDER_ALPHA + (int)( fractionDoneAppearing * (255-WIZROBE_WANDER_ALPHA) );
		m_color = Rgba( 255, 255, 255, alpha );
		m_isHitByPlayerBullets = true;

		/// See if it's time to stop appearing yet
		if( timeNow > m_genericTimeStamp )
		{
			m_genericStateEnum = WIZROBE_STATE_SHOOTING;
			m_genericTimeStamp = timeNow + WIZROBE_SECONDS_TO_SHOOT;
			m_genericTimeStamp2 = timeNow;
		}
	}
	else if( m_genericStateEnum == WIZROBE_STATE_SHOOTING )
	{
		//-------------------------------------------------------------------------------------------------
		// SHOOTING
		//-------------------------------------------------------------------------------------------------
		m_color = Rgba( 255, 255, 255, 255 );
		m_isHitByPlayerBullets = true;

		/// See if it's time to stop appearing yet
		if( timeNow > m_genericTimeStamp )
		{
			m_genericStateEnum = WIZROBE_STATE_DISAPPEARING;
			m_genericTimeStamp = timeNow + WIZROBE_SECONDS_TO_DISAPPEAR;
		}

		/// See if it's time to shoot again yet
		if( timeNow > m_genericTimeStamp2 )
		{
			m_genericTimeStamp2 = timeNow + WIZROBE_SECONDS_BETWEEN_SHOTS; // next shoot time
			WizrobeShoot();
		}
	}
	else if( m_genericStateEnum == WIZROBE_STATE_DISAPPEARING )
	{
		//-------------------------------------------------------------------------------------------------
		// DISAPPEARING
		//-------------------------------------------------------------------------------------------------
		double timeStartDisappearing = m_genericTimeStamp - WIZROBE_SECONDS_TO_DISAPPEAR;
		double timeFinishDisappearing = m_genericTimeStamp;
		double fractionDoneDisappearing = (timeNow - timeStartDisappearing) / (timeFinishDisappearing - timeStartDisappearing);
		fractionDoneDisappearing = 1.0 - ClampDouble( fractionDoneDisappearing, 0.0f, 1.0f );
		int alpha = WIZROBE_WANDER_ALPHA + (int)( fractionDoneDisappearing * (255-WIZROBE_WANDER_ALPHA) );
		m_color = Rgba( 255, 255, 255, alpha );
		m_isHitByPlayerBullets = true;

		/// See if it's time to stop appearing yet
		if( timeNow > m_genericTimeStamp )
		{
			m_genericStateEnum = WIZROBE_STATE_WANDER;
			m_genericTimeStamp = timeNow + (double) RandomFloat( (float)WIZROBE_WANDER_SECONDS_MIN, (float)WIZROBE_WANDER_SECONDS_MAX );
		}
	}
	else if( m_genericStateEnum == WIZROBE_STATE_STUNNED )
	{
		//-------------------------------------------------------------------------------------------------
		// STUNNED
		//-------------------------------------------------------------------------------------------------

		/// Wander around stunned
		const double turnSpeed = m_rotVel * theApplication.GetFrameSeconds();
		const double moveSpeed = WIZROBE_MOVE_TILES_PER_SECOND_WHILE_STUNNED * theApplication.GetFrameSeconds();

		m_color = Rgba( 200, 255, 0, 255 );

		/// Constant rotation
		m_rot += (float) turnSpeed;
		m_rot = fmod( m_rot, 360.0 );

		/// Build move vector
		Vector2 moveVector;
		moveVector.SetFromRadiusCosSinDegrees( moveSpeed, m_rot );
		bool success = TryToMove( moveVector );
		if( !success )
		{
			m_rot += 30.0f;
		}

		/// See if it's time to become unstunned
		if( timeNow > m_genericTimeStamp )
		{
			m_genericStateEnum = WIZROBE_STATE_DISAPPEARING;
			m_genericTimeStamp = timeNow + WIZROBE_SECONDS_TO_DISAPPEAR;
			m_stunned = false;
		}
	}
	else
	{
		m_genericStateEnum = WIZROBE_STATE_WANDER;
	}

	/// See if it's time to die
	double age = timeNow - m_birthTime;
	if( age > WIZROBE_LIFESPAN_SECONDS )
	{
		m_readyToDie = true;
	}
}


//-------------------------------------------------------------------------------------------------
void Entity::Die_EnemyWizrobe()
{
	int randomItemSpawn = RandomInt( 0 , 5 );
	if( randomItemSpawn == 1 )
	{
		SpawnNewEntity( ENT_ITEMPICKUP_MANA, m_pos );
	}
	else
	{
		randomItemSpawn = RandomInt( 0 , 99 );
		if( randomItemSpawn == 1 )
		{
			SpawnNewEntity( ENT_ITEMPICKUP_INVISIBILITY, m_pos );
		}
	}

	delete m_anim ;

	Particle* p = Particle::Spawn();
	p->m_pos = m_pos;
	p->m_rotVel = RandomFloat( 400.0f, 800.0f );
	if( RandomInt(2) )
	{
		p->m_rotVel *= -1.0f;
	}
	p->m_startAlpha = 1.0f;
	p->m_endAlpha = 0.0f;
	p->m_startScale = 1.0f;
	p->m_endScale = 5.0f;
	p->m_timeToLive = 0.4;
	p->m_anim = new Animation( "data/gfx/Enemies/Wizard01.tga" );
}



//-------------------------------------------------------------------------------------------------
// WizrobeShoot
//-------------------------------------------------------------------------------------------------
void Entity::WizrobeShoot()
{
	double timeNow = theApplication.GetTimeFrameBegan();

	/// Spawn a new wizrobe bullet aimed at the player
	Entity& ent = SpawnNewEntity( ENT_BULLET_WIZROBE, m_pos );
	m_timeOfLastSpawn = timeNow;
}



