//-------------------------------------------------------------------------------------------------
// Entity_ItemPickupInvisibility.cpp
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#include "pch.hpp"
#include "engineAssert.hpp"
#include "Entity.hpp"
#include "Particle.hpp"
#include "TheGame.hpp"

const double ITEMPICKUP_INVISIBILITY_LIFESPAN_SECONDS = 10.0;

//-------------------------------------------------------------------------------------------------
// Spawn_ItemPickupInvisibility
//-------------------------------------------------------------------------------------------------
void Entity::Spawm_ItemPickupInvisibility()
{
	theConsole.Printf( "Spawned a Invisibility Pickup Item at %.02f,%.02f\n", m_pos.x, m_pos.y );
	m_anim = new Animation( "data/GFX/Hud/InventoryItemIcon_Invisibility.tga" );
	m_scale.SetXY( 0.4f, 0.4f );
	m_bounds.SetFromMinXYMaxXY( -8.0f, -8.0f, 8.0f, 8.0f );
	m_rotVel = 0.0f;
}


//-------------------------------------------------------------------------------------------------
// Think_ItemPickupInvisibility
//-------------------------------------------------------------------------------------------------
void Entity::Think_ItemPickupInvisibility()
{
	double timeNow = theApplication.GetTimeFrameBegan();
	double age = timeNow - m_birthTime;

	/// Check if we hit the player
	if( theGame->IsPointWithinGivenRadiusOfPlayer( m_pos, 1.0f ) )
	{
		theGame->m_player->m_inventory.GiveItem( "Invisibility", 1 );
		m_readyToDie = true;
	}

	if( age > ITEMPICKUP_INVISIBILITY_LIFESPAN_SECONDS )
	{
		m_readyToDie = true;
	}
}


//-------------------------------------------------------------------------------------------------
// Die_ItemPickupInvisibility
//-------------------------------------------------------------------------------------------------
void Entity::Die_ItemPickupInvisibility()
{
	delete m_anim ;

	Particle* p = Particle::Spawn();
	p->m_pos = m_pos;
	p->m_startAlpha = 1.0f;
	p->m_endAlpha = 0.0f;
	p->m_startScale = 0.4f;
	p->m_endScale = 1.0;
	p->m_timeToLive = 0.35;
	p->m_anim = new Animation( "data/GFX/Hud/InventoryItemIcon_Invisibility.tga" );
}



