// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.

#include "pch.hpp"
#include "engineAssert.hpp"
#include "Entity.hpp"
#include "Particle.hpp"
#include "TheGame.hpp"

const float PLAYER_SPELL1_DAMAGE = 20.0f;

//-------------------------------------------------------------------------------------------------
void Entity::Spawn_PlayerSpell1()
{
	m_bounds.SetFromMinXYMaxXY( -8.0f, -8.0f, 8.0f, 8.0f );

	m_anim			= new Animation( Player::PLAYER_SPELL1 );

	/*
	Particle* p = Particle::Spawn();
	p->m_pos		= m_pos;
	p->m_startAlpha	= 0.8f;
	p->m_endAlpha	= 0.8f;
	p->m_startScale	= 1.0f;
	p->m_endScale	= 1.0ff;
	p->m_timeToLive	= 10.0f;
	p->m_anim		= new Animation( Player::PLAYER_SPELL1 );
	*/
}


//-------------------------------------------------------------------------------------------------
void Entity::Think_PlayerSpell1()
{
	const double elapsed	= theApplication.GetFrameSeconds();
	const double lifetime	= (theApplication.GetTimeFrameBegan() - m_birthTime);

	Vector2	entPos	= m_pos ;
	Vector2	fwd;

	fwd.SetFromCosSinDegrees( m_rot );
	entPos += fwd * (float)elapsed * 12.0f ;
	m_pos	= entPos ;

	AABB2 otherAbsBounds;
	AABB2 absBounds	 = m_bounds;
	absBounds		+= m_pos * 32;


	MapData&	mapData		= *theGame->m_currentMap;
	Material&	material	= mapData.GetMaterialForMapXY( (int)m_pos.x, (int)m_pos.y );

	if ( material.m_flags & (MATFLAG_BLOCKS_PROJECTILES) )
	{
		m_readyToDie = true ;
	}

	
	std::list< Entity* >::iterator end = mapData.entityList.end();
	for( std::list< Entity* >::iterator cur = mapData.entityList.begin(); cur != end; cur++ )
	{
		Entity* e = *cur;
		if( e->m_isHitByPlayerBullets )
		{
			otherAbsBounds = e->m_bounds;
			otherAbsBounds += e->m_pos * 32;
			if( otherAbsBounds.IsOverlapping( absBounds ) )
			{
				e->DealDamage( PLAYER_SPELL1_DAMAGE );
				e->m_isBeingDamagedByPlayer = true;
				m_readyToDie = true;
			}
		}
	}
}


//-------------------------------------------------------------------------------------------------
void Entity::Die_PlayerSpell1()
{
	delete m_anim;
	Particle* p = Particle::Spawn();
	p->m_pos = m_pos;
	p->m_startAlpha = 1.0f;
	p->m_endAlpha = 0.0f;
	p->m_startScale = 1.0f;
	p->m_endScale = 5.0f;
	p->m_timeToLive = 0.3;
	p->m_anim = new Animation( "data/gfx/spells/spell02expl.tga" );

	DamageMapGridRadius( m_pos, 1.0f , false );
}
