// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.

#include "pch.hpp"
#include "engineAssert.hpp"
#include "Entity.hpp"
#include "Particle.hpp"
#include "TheGame.hpp"

const float PLAYER_SPELL2_MOVE_TILES_PER_SECOND = 8.0f;
const double PLAYER_SPELL2_LIFESPAN_SECONDS = 1.0;
const float PLAYER_SPELL2_DAMAGE = 5.0f;

//-------------------------------------------------------------------------------------------------
void Entity::Spawn_PlayerSpell2()
{
	m_bounds.SetFromMinXYMaxXY( -8.0f, -8.0f, 8.0f, 8.0f );
	m_rotVel = RandomFloat( -10.0f, 10.0f );
	m_drawRotated = true;
	m_scale.SetXY( .25, .25 );
	m_color = Rgba( 255, 255, 255, 255 );
	m_anim	= new Animation( Player::PLAYER_SPELL2 );

	m_rot = theGame->m_player->rotDeg;
	m_rot += RandomFloat( -10.0f, 10.0f);
	m_vel.SetFromCosSinDegrees( m_rot );
}


//-------------------------------------------------------------------------------------------------
void Entity::Think_PlayerSpell2()
{
	double timeNow = theApplication.GetTimeFrameBegan();
	double age = timeNow - m_birthTime;

	float moveSpeed = (float)( PLAYER_SPELL2_MOVE_TILES_PER_SECOND * theApplication.GetFrameSeconds() );
	Vector2 moveVector = m_vel * moveSpeed;
	m_scale.x = m_scale.x + 0.05;
	m_scale.y = m_scale.y + 0.05;
	m_color.a = m_color.a * 0.9;
	m_rot += m_rotVel;

	m_pos += moveVector ;

	AABB2 otherAbsBounds;
	AABB2 absBounds	 = m_bounds;
	absBounds		+= m_pos * 32;

	MapData&	mapData		= *theGame->m_currentMap;
	Material&	material	= mapData.GetMaterialForMapXY( (int)m_pos.x, (int)m_pos.y );

	if ( material.m_flags & (MATFLAG_BLOCKS_PROJECTILES) )
	{
		m_readyToDie = true ;
	}

	/// See if it's time to die
	if( age > PLAYER_SPELL2_LIFESPAN_SECONDS )
	{
		m_readyToDie = true;
	}
	
	std::list< Entity* >::iterator end = mapData.entityList.end();
	for( std::list< Entity* >::iterator cur = mapData.entityList.begin(); cur != end; cur++ )
	{
		Entity* e = *cur;
		if( e->m_isHitByPlayerBullets )
		{
			otherAbsBounds = e->m_bounds;
			otherAbsBounds += e->m_pos * 32;
			if( otherAbsBounds.IsOverlapping( absBounds ) )
			{
				if( e->m_bossStage > 0 )
				{
					e->DealDamage( PLAYER_SPELL2_DAMAGE * 0.35 );
				}
				else
				{
					e->DealDamage( PLAYER_SPELL2_DAMAGE );
				}

				e->m_isBeingDamagedByPlayer = true;
			}
		}
	}
}


//-------------------------------------------------------------------------------------------------
void Entity::Die_PlayerSpell2()
{
	delete m_anim;
}
