// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.

#include "pch.hpp"
#include "engineAssert.hpp"
#include "Entity.hpp"
#include "Particle.hpp"
#include "TheGame.hpp"

const float PLAYER_SPELL3_DAMAGE = 1.0f;

//-------------------------------------------------------------------------------------------------
void Entity::Spawn_PlayerSpell3()
{
	m_bounds.SetFromMinXYMaxXY( -8.0f, -8.0f, 8.0f, 8.0f );

	Particle* p = Particle::Spawn();
	p->m_pos = m_pos;
	p->m_startAlpha = 0.6f;
	p->m_endAlpha = 0.2f;
	p->m_startScale = 0.9f;
	p->m_endScale = 0.01f;
	p->m_timeToLive = 0.2;
	p->m_anim = new Animation( Player::PLAYER_SPELL3 );
}

//-------------------------------------------------------------------------------------------------
void Entity::Think_PlayerSpell3()
{
	const double elapsed = (theApplication.GetTimeFrameBegan() - m_birthTime);
	if( m_genericCounter > 0 && m_timeOfLastSpawn == 0.0 && elapsed > 0.01 )
	{
		Vector2 entPos = m_pos;
		Vector2 fwd;
		fwd.SetFromCosSinDegrees( m_rot );
		entPos += fwd * 0.1f;
		Entity& e = Entity::SpawnNewEntity( ENT_PLAYER_SPELL3, entPos );
		e.m_genericCounter = m_genericCounter - 1;
		e.m_rot = m_rot;
		m_timeOfLastSpawn = 1.0;
		return;
	}

	if( elapsed > 0.2 )
	{
		m_readyToDie = true;
		return;
	}

	AABB2 absBounds = m_bounds;
	absBounds += m_pos * 32;
	AABB2 otherAbsBounds;
	MapData& mapData = *theGame->m_currentMap;
	std::list< Entity* >::iterator end = mapData.entityList.end();
	for( std::list< Entity* >::iterator cur = mapData.entityList.begin(); cur != end; cur++ )
	{
		Entity* e = *cur;

		if( !e->m_isHitByPlayerBullets )
			continue;

		otherAbsBounds = e->m_bounds;
		otherAbsBounds += e->m_pos * 32;
		if( otherAbsBounds.IsOverlapping( absBounds ) )
		{

			if( e->m_type == ENT_ENEMY_BOSS1 )
			{
				e->DealDamage( PLAYER_SPELL3_DAMAGE * 0.75 );
			}
			else
			{
				e->DealDamage( PLAYER_SPELL3_DAMAGE );
			}
			e->m_isBeingDamagedByPlayer = true;
		}
	}
}

//-------------------------------------------------------------------------------------------------
void Entity::Die_PlayerSpell3()
{
	delete m_anim;
}
