//-------------------------------------------------------------------------------------------------
// Inventory.cpp (game)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
// 
// This source may not be used, modified, or distributed
//  without the express written permission of Brian 'Squirrel' Eiserloh.
//-------------------------------------------------------------------------------------------------
#include "pch.hpp"
#include "Inventory.hpp"
#include "TheGame.hpp"






//-------------------------------------------------------------------------------------------------
// Constructor
//-------------------------------------------------------------------------------------------------
Inventory::Inventory()
:
m_numKey( 0 ),
m_numBomb( 0 ),
m_numStunner( 0 ),
m_numHealth( 0 ),
m_numMana( 0 ),
m_numInvisibility( 0 ),
m_numOrb( 0 ),
m_activeInventoryItemNumber( 0 )
{
}


//-------------------------------------------------------------------------------------------------
// Destructor
//-------------------------------------------------------------------------------------------------
Inventory::~Inventory()
{
}



//-------------------------------------------------------------------------------------------------
// GiveItem
//-------------------------------------------------------------------------------------------------
void Inventory::GiveItem( const char* itemName, int itemCount )
{
	if( !stricmp( itemName, "key" ) )
	{
		m_numKey += itemCount;
		m_numKey = ClampInt( m_numKey, 0, 99 );
	}
	else if( !stricmp( itemName, "bomb" ) )
	{
		m_numBomb += itemCount;
		m_numBomb = ClampInt( m_numBomb, 0, 99 );
	}
	else if( !stricmp( itemName, "stunner" ) )
	{
		m_numStunner += itemCount;
		m_numStunner = ClampInt( m_numStunner, 0, 99 );
	}
	else if( !stricmp( itemName, "health" ) )
	{
		m_numHealth += itemCount;
		m_numHealth = ClampInt( m_numHealth, 0, 99 );
	}
	else if( !stricmp( itemName, "mana" ) )
	{
		m_numMana += itemCount;
		m_numMana = ClampInt( m_numMana, 0, 99 );
	}
	else if( !stricmp( itemName, "invisibility" ) )
	{
		m_numInvisibility += itemCount;
		m_numInvisibility = ClampInt( m_numInvisibility, 0, 99 );
	}
	else if( !stricmp( itemName, "orb" ) )
	{
		m_numOrb += itemCount;
		m_numOrb = ClampInt( m_numOrb, 0, 9 );
	}
	else if( !stricmp( itemName, "all" ) )
	{
		m_numBomb += 99;
		m_numBomb = ClampInt( m_numBomb, 0, 99 );
		m_numStunner += 99;
		m_numStunner = ClampInt( m_numStunner, 0, 99 );
		m_numHealth += 99;
		m_numHealth = ClampInt( m_numHealth, 0, 99 );
		m_numMana += 99;
		m_numMana = ClampInt( m_numMana, 0, 99 );
		m_numInvisibility += 99;
		m_numInvisibility = ClampInt( m_numInvisibility, 0, 99 );
	}
	else if( !stricmp( itemName, "random" ) )
	{

		float randomItem = RandomInt( 1 , 13 );

		if( randomItem == 1 || randomItem == 2 || randomItem == 3 )
		{
			m_numBomb += itemCount;
			m_numBomb = ClampInt( m_numBomb, 0, 99 );
		}
		else if( randomItem == 4 || randomItem == 5 || randomItem == 6  )
		{
			m_numStunner += itemCount;
			m_numStunner = ClampInt( m_numStunner, 0, 99 );
		}
		else if( randomItem == 7 || randomItem == 8 || randomItem == 9  )
		{
			m_numHealth += itemCount;
			m_numHealth = ClampInt( m_numHealth, 0, 99 );
		}
		else if( randomItem == 10 || randomItem == 11 || randomItem == 12  )
		{
			m_numMana += itemCount;
			m_numMana = ClampInt( m_numMana, 0, 99 );
		}
		else if( randomItem == 13 )
		{
			m_numInvisibility += itemCount;
			m_numInvisibility = ClampInt( m_numInvisibility, 0, 99 );
		}
	}
}


//-------------------------------------------------------------------------------------------------
// RemoveItem
//-------------------------------------------------------------------------------------------------
void Inventory::RemoveItem( const char* itemName, int itemCount )
{
	if( !stricmp( itemName, "key" ) )
	{
		m_numKey -= itemCount;
		m_numKey = ClampInt( m_numKey, 0, 99 );
	}
	else if( !stricmp( itemName, "bomb" ) )
	{
		m_numBomb -= itemCount;
		m_numBomb = ClampInt( m_numBomb, 0, 99 );
	}
	else if( !stricmp( itemName, "stunner" ) )
	{
		m_numStunner -= itemCount;
		m_numStunner = ClampInt( m_numStunner, 0, 99 );
	}
	else if( !stricmp( itemName, "health" ) )
	{
		m_numHealth -= itemCount;
		m_numHealth = ClampInt( m_numHealth, 0, 99 );
	}
	else if( !stricmp( itemName, "mana" ) )
	{
		m_numMana -= itemCount;
		m_numMana = ClampInt( m_numMana, 0, 99 );
	}
	else if( !stricmp( itemName, "invisibility" ) )
	{
		m_numInvisibility -= itemCount;
		m_numInvisibility = ClampInt( m_numInvisibility, 0, 99 );
	}
	else if( !stricmp( itemName, "orb" ) )
	{
		m_numOrb -= itemCount;
		m_numOrb = ClampInt( m_numOrb, 0, 9 );
	}
	else if( !stricmp( itemName, "all" ) )
	{
		m_numBomb -= 99;
		m_numBomb = ClampInt( m_numBomb, 0, 99 );
		m_numStunner -= 99;
		m_numStunner = ClampInt( m_numStunner, 0, 99 );
		m_numHealth -= 99;
		m_numHealth = ClampInt( m_numHealth, 0, 99 );
		m_numMana -= 99;
		m_numMana = ClampInt( m_numMana, 0, 99 );
		m_numInvisibility -= 99;
		m_numInvisibility = ClampInt( m_numInvisibility, 0, 99 );
	}
}


//-------------------------------------------------------------------------------------------------
// UseCurrentActiveItem
//-------------------------------------------------------------------------------------------------
void Inventory::UseCurrentActiveItem()
{
	if( m_activeInventoryItemNumber == ACTIVE_INVENTORY_ITEM_BOMB )
	{
		theGame->m_player->PlantBomb();
	}
	else if( m_activeInventoryItemNumber == ACTIVE_INVENTORY_ITEM_STUNNER )
	{
		theGame->m_player->UseStunner();
	}
	else if( m_activeInventoryItemNumber == ACTIVE_INVENTORY_ITEM_HEALTH )
	{
		theGame->m_player->DrinkHealthPotion();
	}
	else if( m_activeInventoryItemNumber == ACTIVE_INVENTORY_ITEM_MANA )
	{
		theGame->m_player->DrinkManaPotion();
	}
	else if( m_activeInventoryItemNumber == ACTIVE_INVENTORY_ITEM_INVISIBILITY )
	{
		theGame->m_player->DrinkInvisibilityPotion();
	}
}


//-------------------------------------------------------------------------------------------------
// RotateCurrentActiveItem
//-------------------------------------------------------------------------------------------------
void Inventory::RotateCurrentActiveItem()
{
	m_activeInventoryItemNumber --;
	m_activeInventoryItemNumber += NUM_ACTIVE_INVENTORY_ITEMS;
	m_activeInventoryItemNumber %= NUM_ACTIVE_INVENTORY_ITEMS;
}


