
#include "pch.hpp"
#include "lights.hpp"

Lights::Lights()
:	_attenuationTexture( 0 ),
	m_mapRadiusScale( 1.0f ),
	m_playerRadiusScale( 1.0f ),
	m_fogColor( Rgba::BLACK )
{
}

Lights::~Lights()
{
}

void Lights::setTexture( Texture* const tex )
{
	_attenuationTexture = tex;
}

void Lights::draw()
{
	Vector2 offset( 256, 0 );
	theRenderer.TranslateCoordinateSystem( offset );

	Vector2 center( 384, 384 );
	Vector2 size( 768, 768 );
	Vector2 texTopLeft( 0.0, 1.0 );
	Vector2 texBottomRight( 1.0, 0.0 );
	
	const float scale = m_mapRadiusScale * m_playerRadiusScale;
	float centerOffset = scale * 384;

	Vector2 topLeft( center.x - centerOffset, center.y - centerOffset );
	Vector2 bottomRight( center.x + centerOffset, center.y + centerOffset );

	glBlendFunc( GL_ONE_MINUS_SRC_ALPHA, GL_SRC_ALPHA );
	glEnable( GL_TEXTURE_2D );

	theRenderer.SetCurrentTexture( _attenuationTexture->GetGLTexNum() );
	theRenderer.SetColor( m_fogColor );

	// draw fog
	glBegin( GL_QUADS );
		
		glTexCoord2f( texTopLeft.x, texTopLeft.y );
		glVertex2f( topLeft.x, topLeft.y );

		glTexCoord2f( texTopLeft.x, texBottomRight.y );
		glVertex2f( topLeft.x, bottomRight.y );

		glTexCoord2f( texBottomRight.x, texBottomRight.y );
		glVertex2f( bottomRight.x, bottomRight.y );

		glTexCoord2f( texBottomRight.x, texTopLeft.y );
		glVertex2f( bottomRight.x, topLeft.y );

	glEnd();

	// draw dark
	glDisable( GL_TEXTURE_2D );
	glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );

	if( scale < 1.0 )
	{
		glBegin( GL_QUAD_STRIP );

			glVertex2f( topLeft.x, topLeft.y );
			glVertex2f( 0, 0 );
			glVertex2f( bottomRight.x, topLeft.y );
			glVertex2f( 768, 0 );
			
			glVertex2f( bottomRight.x, bottomRight.y );
			glVertex2f( 768, 768 );

			glVertex2f( topLeft.x, bottomRight.y );
			glVertex2f( 0, 768 );

			glVertex2f( topLeft.x, topLeft.y );
			glVertex2f( 0, 0 );

		glEnd();
	}

	theRenderer.TranslateCoordinateSystem( -offset );
}
