//-------------------------------------------------------------------------------------------------
// MapData.hpp (engine)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#ifndef _include_MapData_
#define _include_MapData_


//-------------------------------------------------------------------------------------------------
// Included header files
//-------------------------------------------------------------------------------------------------
#include "engineCommon.hpp"
#include "engineAssert.hpp"
#include "Material.hpp"
#include "Image.hpp"
#include "Entity.hpp"


//-------------------------------------------------------------------------------------------------
// Class Constants
//-------------------------------------------------------------------------------------------------
const int MAP_TILE_SIZE_X = 96;
const int MAP_TILE_SIZE_Y = 96;
const int TILE_PIXEL_SIZE_X = 32;
const int TILE_PIXEL_SIZE_Y = 32;



///////////////////////////////////////////////////////////////////////////////////////////////////
//
//		MapTileData
//
///////////////////////////////////////////////////////////////////////////////////////////////////
struct MapTileData
{
	int				m_materialID;
	unsigned int	m_debugFrameCountMark;
};

///////////////////////////////////////////////////////////////////////////////////////////////////
//
//		MapData
//
///////////////////////////////////////////////////////////////////////////////////////////////////
class MapData
{
public:
	MapData( const Image& mapDataImage, const char* mapName );
	~MapData();

	static void						LoadAllMaps();
	static bool						LoadMap( const char* const mapSet, const int mapNumber );
	void							SpawnAllEntities();
	Material&						GetMaterialForMapXY( int x, int y );
	IntVector2						FindFirstTileWithMaterialID( int materialID );
	void							ChangeMaterialAtPosition( const IntVector2& tilePos, const std::string& materialName );
	void							ParseMetaData( const char* const mapSet, const int mapNumber );

	float							m_lightScale;
	Rgba							m_fogColor;
	
	MapTileData						tileArray[ MAP_TILE_SIZE_X ][ MAP_TILE_SIZE_Y ];
	std::list< Entity* >			entityList;
	static std::vector<MapData*>	s_mapDataList;
};




#endif // _include_MapData_
