//-------------------------------------------------------------------------------------------------
// MapDisplay.cpp (game)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#include "pch.hpp"
#include "engineAssert.hpp"
#include "MapDisplay.hpp"
#include "MapData.hpp"
#include "TheGame.hpp"

MapDisplay::MapDisplay()
:
_tileTexture( 0 ),
m_isDebugDrawingEnabled( false )
{
}


MapDisplay::~MapDisplay()
{
	_UnloadTiles();
}


void MapDisplay::LoadTiles( const std::string tilesetName )
{
	_UnloadTiles();
	Image img( "data/maps/" + tilesetName + ".tga" );
	_tileTexture = new Texture( img );
}

void MapDisplay::_UnloadTiles()
{
	if( _tileTexture )
	{
		delete _tileTexture;
		_tileTexture = 0;
	}
}

void MapDisplay::Draw( int mapNumber )
{
	ASSERT( _tileTexture );

	MapData* mapdata = MapData::s_mapDataList[ mapNumber ];
	
	//Vector2 bottomRight( (float) theVideoSystem.GetCurrentVidModeWidth() - 256, (float) theVideoSystem.GetCurrentVidModeHeight() );
	//Vector2 topLeft( -256, 0 );
	//theRenderer.SetCoordinateSystem( topLeft, bottomRight );
	glTranslated( 256, 0, 0 );

	glEnable( GL_TEXTURE_2D );
	glShadeModel( GL_FLAT );
	glColor3f( 1.0f, 1.0f, 1.0f );
	
	theRenderer.SetCurrentTexture( _tileTexture->GetGLTexNum() );

	glBegin( GL_QUADS );

	const double xofs = (int)((theGame->m_player->pos.x - ((int) theGame->m_player->pos.x )) * -32);
	const double yofs = (int)((theGame->m_player->pos.y - ((int) theGame->m_player->pos.y )) * -32);

	const int tileLeft	= ((int) theGame->m_player->pos.x) - 12;
	const int tileTop	= ((int) theGame->m_player->pos.y) - 12;
	
	for( int y = -1; y < 25; y++ )
	{
		for( int x = -1; x < 25; x++ )
		{
			const int tileX = tileLeft + x;
			const int tileY = tileTop + y;
			
			if( tileX < 0 )					continue;
			if( tileY < 0 )					continue;
			if( tileX >= MAP_TILE_SIZE_X )	continue;
			if( tileY >= MAP_TILE_SIZE_Y )	continue;

			Material& mat = mapdata->GetMaterialForMapXY( tileX, tileY );

			const double texCoordLeft		= mat.m_mapGfxTexCoordsTopLeft.x;
			const double texCoordTop		= 1.0 - mat.m_mapGfxTexCoordsTopLeft.y;
			const double texCoordRight		= texCoordLeft + 1.0/16.0;
			const double texCoordBottom		= texCoordTop - 1.0/16.0;

			const double vertLeft = xofs + x * 32;
			const double vertTop = yofs + y * 32;
			const double vertRight = vertLeft + 32;
			const double vertBottom = vertTop + 32;

			SetTileDrawColor( *mapdata, tileX, tileY, mat );

			glTexCoord2d( texCoordLeft, texCoordTop );
			glVertex2d( vertLeft, vertTop );

			glTexCoord2d( texCoordRight, texCoordTop );
			glVertex2d( vertRight, vertTop );

			glTexCoord2d( texCoordRight, texCoordBottom );
			glVertex2d( vertRight, vertBottom );

			glTexCoord2d( texCoordLeft, texCoordBottom );
			glVertex2d( vertLeft, vertBottom );

//			DrawTileDebugInfo( tileX, tileY, mat, AABB2( (float) vertLeft, (float) vertTop, (float) vertRight, (float) vertBottom ) );
		}
	}

	glEnd();

	glTranslatef( -256, 0, 0 );


}



//-------------------------------------------------------------------------------------------------
// SetTileDrawColor
//-------------------------------------------------------------------------------------------------
void MapDisplay::SetTileDrawColor( MapData& mapData, int tileX, int tileY, const Material& mat )
{
	/// Handle general case
	glColor3f( 1.0f, 1.0f, 1.0f );
	if( !m_isDebugDrawingEnabled )
		return;

	unsigned int frameNum = theApplication.GetFrameNumber();
	double timeNow = theApplication.GetTimeFrameBegan();
	float voob1 = (float) fabs( cos( timeNow * 30.0 ) );

	if( mapData.tileArray[ tileX ][ tileY ].m_debugFrameCountMark == frameNum )
	{
		glColor3f( 1.0f, voob1, voob1 );
	}
}


