//-------------------------------------------------------------------------------------------------
// Material.hpp (engine)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#ifndef _include_Material_
#define _include_Material_


//-------------------------------------------------------------------------------------------------
// Included header files
//-------------------------------------------------------------------------------------------------
#include "gameCommon.hpp"
#include "engineAssert.hpp"
#include "ParseFile.hpp"
#include "Rgba.hpp"
#include "IntVector2.hpp"
#include "Entity.hpp"
#include "Scripting.hpp"


//-------------------------------------------------------------------------------------------------
// Materials properties flags
//-------------------------------------------------------------------------------------------------
enum
{
	MATFLAG_BLOCKS_PLAYERS		= BIT( 0 ),
	MATFLAG_BLOCKS_ENEMIES		= BIT( 1 ),
	MATFLAG_BLOCKS_PROJECTILES	= BIT( 2 ),
};


///////////////////////////////////////////////////////////////////////////////////////////////////
//
//		Material
//
///////////////////////////////////////////////////////////////////////////////////////////////////
class Material
{
public:
	/// Static data
	static Material									s_missingMaterial;
	static std::map< std::string, TypeOfEntity >	s_entityTypeMap;

	/// Member data
	std::string		m_name;
	Rgba			m_color;
	IntVector2		m_mapGfxTileCoords;
	Vector2			m_mapGfxTexCoordsTopLeft;
	int				m_id;
	unsigned int	m_flags;
	TypeOfEntity	m_spawnEntityType;
	char*			m_replacementMaterialOnSpawn;
	std::list<ScriptEventHandler>	m_eventsHandled;

	/// Static methods
	static void BuildMaterialTable();
	static void ParseColorKeyTable( ParseFile& parseFile );
	static void ParseMaterialDescription( ParseFile& parseFile );
	static void ParseMaterialData( ParseFile& parseFile, const char* name );
	static void ParseMaterialKey_mapgfxtilecoords( ParseFile& parseFile, Material& material, const char* keyName );
	static void ParseMaterialKey_impassable( ParseFile& parseFile, Material& material, const char* keyName );
	static void ParseMaterialKey_blocksplayers( ParseFile& parseFile, Material& material, const char* keyName );
	static void ParseMaterialKey_blocksmonsters( ParseFile& parseFile, Material& material, const char* keyName );
	static void ParseMaterialKey_blocksprojectiles( ParseFile& parseFile, Material& material, const char* keyName );
	static void ParseMaterialKey_spawn( ParseFile& parseFile, Material& material, const char* keyName );
	static void ParseMaterialKey_replacewith( ParseFile& parseFile, Material& material, const char* keyName );
	static void ParseMaterialKey_event( ParseFile& parseFile, Material& material, const char* keyName );
	static void ParseEventAction( ParseFile& parseFile, ScriptEventHandler& eventHandler );
	static Material& FindMaterialByName( const char* name );
	static Material& FindMaterialByColor( const Rgba& color );
	static Material& FindMaterialByID( int id );
	static void Init();

	/// Member methods
	Material( const std::string& name, const Rgba& color, int materialID );
	Material( const Material& copy );
	Material();
	~Material();
	void PrintDebugInfo();
	ScriptEventHandler* GetEventHandlerForEvent( std::string eventTypeName );
};



//-------------------------------------------------------------------------------------------------
// extern'd global data
//-------------------------------------------------------------------------------------------------
const int MAX_NUM_MATERIALS = 256;
extern Material g_materialTable[ MAX_NUM_MATERIALS ];
extern int g_numMaterials;


#endif // _include_Material_
