// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.

#include "pch.hpp"
#include "engineAssert.hpp"
#include "Particle.hpp"

STATIC std::list< Particle* > Particle::s_particleList;

Particle::Particle()
:
m_anim( 0 ),
m_rot( 0.0f ),
m_rotVel( 0.0f )
{
}

Particle::~Particle()
{
	if( m_anim )
		delete m_anim;
}

void Particle::UpdateAll()
{
	const double timeFraction = theApplication.GetFrameSeconds();
	const double time = theApplication.GetTimeFrameBegan();
	std::list< Particle* >::iterator start = s_particleList.begin();
	std::list< Particle* >::iterator end = s_particleList.end();
	std::list< Particle* >::iterator next = start;
	for( std::list< Particle* >::iterator cur = start; cur != end; cur = next )
	{
		++next;
		Particle* p = *cur;
		p->m_rot += p->m_rotVel * (float)timeFraction;
		if( p->m_birthTime + p->m_timeToLive < time )
		{
			s_particleList.erase( cur );
			delete p;
		}
	}
}

void Particle::DrawAll()
{
	const double time = theApplication.GetTimeFrameBegan();
	Vector2 scalevec;
	Rgba color;

	std::list< Particle* >::iterator start = s_particleList.begin();
	std::list< Particle* >::iterator end = s_particleList.end();
	for( std::list< Particle* >::iterator cur = start; cur != end; ++cur )
	{
		Particle* p = *cur;
		const float timeFrac = (float)((time - p->m_birthTime) / p->m_timeToLive);
		const float alpha = p->m_startAlpha + ((p->m_endAlpha - p->m_startAlpha) * timeFrac);
		const float scale = p->m_startScale + ((p->m_endScale - p->m_startScale) * timeFrac);
		scalevec.SetXY( scale, scale );
		color.SetRGBAFloats( 1.0, 1.0, 1.0, alpha );
		p->m_anim->Draw( p->m_pos * 32, scalevec, p->m_rot, color );
	}
}
