//-------------------------------------------------------------------------------------------------
// Player.cpp (game)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
// 
// This source may not be used, modified, or distributed
//  without the express written permission of The Universe.
// Damn it feels good to be a gangsta.
//-------------------------------------------------------------------------------------------------

#include "pch.hpp"
#include "TheGame.hpp"
#include "Player.hpp"







//-------------------------------------------------------------------------------------------------
// Destructor
//-------------------------------------------------------------------------------------------------
Player::~Player()
{}


//-------------------------------------------------------------------------------------------------
// Constructor
//-------------------------------------------------------------------------------------------------
Player::Player()
:
speedMultiplier( 1.0f ),
m_wasHurtThisFrame( false ),
m_invisibilitySeconds( 0.0 ),
m_painLevel( 0.0f ),
m_godMode( false ),
m_noclipMode( false ),
m_difficulty( 1 ),
m_worldBounds( Vector2::ZERO )
{
	// set default player info
	pos.x = 0.5f;	
	pos.y = 0.5f;	
	rotDeg = 0.0f;	
	scale.x = 1;	
	scale.y = 1;
	move_left = -1;	
	move_right = -1;	
	move_up = -1;	
	move_down = -1;

	for( int i = 0; i < NUM_INVENTORY_ITEM_SLOTS; i ++ )
	{
//		m_itemCount[ i ] = 0;
//		m_itemTextures[ i ] = NULL;
	}

	PlayerDrawInit();
}


//-------------------------------------------------------------------------------------------------
// CheckForPlayerDeath
//-------------------------------------------------------------------------------------------------
void Player::CheckForPlayerDeath()
{
	if( health <= 0.0f )
	{
		health = 0.0f;
		theGame->ChangeState( GAMESTATE_GAMEOVER );
	}
}


//-------------------------------------------------------------------------------------------------
// CheckForPlayerWin
//-------------------------------------------------------------------------------------------------
void Player::CheckForPlayerWin()
{
	if( m_inventory.m_numOrb >= NUM_ORBS )
	{
		theGame->ChangeState( GAMESTATE_YOUWIN );
	}
}


//-------------------------------------------------------------------------------------------------
// Update
//-------------------------------------------------------------------------------------------------
void Player::Update()
{
	UpdatePlayerMovement();
	UpdateMagic();
	CheckForPlayerDeath();
	CheckForPlayerWin();
}


//--------------------------------------------------------------------------------------------------
// ToggleGodMode
//--------------------------------------------------------------------------------------------------
bool Player::ToggleGodMode( void )
{
	if ( m_godMode )
	{
		m_godMode = false ;
//		if ( m_noclipMode )
//			ToggleNoClipMode();
	}
	else
	{
		m_godMode	= true ;
//		if ( !m_noclipMode )
//			ToggleNoClipMode();
	}

	return m_godMode ;
}


//--------------------------------------------------------------------------------------------------
// ToggleNoClipMode
//--------------------------------------------------------------------------------------------------
bool Player::ToggleNoClipMode( void )
{
	if ( theGame->m_player->m_noclipMode )
	{
		theGame->m_noClip					= false ;
		theGame->m_player->m_noclipMode		= false ;
		theGame->m_player->speedMultiplier	= 1.0f;
	}
	else
	{
		theGame->m_noClip					= true ;
		theGame->m_player->m_noclipMode		= true ;
		theGame->m_player->speedMultiplier	= 10.0f;
	}

	return m_noclipMode ;
}


//-------------------------------------------------------------------------------------------------
// OnPlayerCommandKey
//-------------------------------------------------------------------------------------------------
void Player::OnPlayerCommandKey( const byte_t key )
{
	// Toggle movement on or off

	// UP
	if ( key == VK_UP || key == VK_NUMPAD8)
		move_up *= -1;
	// DOWN
	if ( key == VK_DOWN || key == VK_NUMPAD2)
		move_down *= -1;
	// LEFT
	if ( key == VK_LEFT || key == VK_NUMPAD4)
		move_left *= -1;
	// RIGHT
	if ( key == VK_RIGHT || key == VK_NUMPAD6)
		move_right *= -1;

	// CAST MAGIC
	if (key == '1')
		CastMagic(1);
	else if (key == '2')
		CastMagic(2);
	else if (key == '3')
		CastMagic(3);
	else if (key == '4')
		CastMagic(4);
	else if (key == '5')
		CastMagic(5);
	else if (key == '6')
		CastMagic(6);
}


///-------------------------------------------------------------------------
/// Teleport
///-------------------------------------------------------------------------
void Player::Teleport( const char* materialName, int delta )
{
	Material& destinationMaterial = Material::FindMaterialByName( materialName );

	int newMapNumber = theGame->m_currentMapNumber + 1+ delta ;
	if ( newMapNumber < 0 || newMapNumber > (int)MapData::s_mapDataList.size() )
		return ;

	MapData* mapData		= MapData::s_mapDataList[ newMapNumber - 1 ];
	IntVector2 mapPosition	= mapData->FindFirstTileWithMaterialID( destinationMaterial.m_id );
	
	theGame->WarpToLevel( newMapNumber, mapPosition.x, mapPosition.y );
}


///---------------------------------------------------------------------------
/// AdjustHealth -- Can be used to take or give health.  If taking (damage)
///                 this will set the damaged flag
///---------------------------------------------------------------------------
void Player::AdjustHealth( float healthDelta )
{
	if ( !m_godMode )
	{
		health += healthDelta ;
		health = ClampFloat( health, 0.0f, 100.0f );
		if ( healthDelta < 0 )
		{
			m_wasHurtThisFrame = true;
			m_painLevel += -healthDelta * 0.03f;
		}
	}
}


///---------------------------------------------------------------------------
/// AdjustMana -- Can be used to take or give mana.
///---------------------------------------------------------------------------
void Player::AdjustMana( float delta )
{
	m_mana += delta;
	m_mana = ClampFloat( m_mana, 0.0f, 100.0f );
}


