//-------------------------------------------------------------------------------------------------
// Player.hpp (game)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
// 
// This source may not be used, modified, or distributed
//  without the express written permission of Brian 'Squirpel' Eiserloh.
//-------------------------------------------------------------------------------------------------
#ifndef _include_Player_
#define _include_Player_


//-------------------------------------------------------------------------------------------------
// Included header files
//-------------------------------------------------------------------------------------------------
#include "gameCommon.hpp"
#include "Inventory.hpp"

enum
{
	NUM_INVENTORY_ITEM_SLOTS = 6,
};


///////////////////////////////////////////////////////////////////////////////////////////////////
//
//		Player
// 
///////////////////////////////////////////////////////////////////////////////////////////////////

class Player
{
private:
public:

	static const char*			PLAYER_ANIM_NAME_IDLE;
	static const char*			PLAYER_ANIM_NAME_UP;
	static const char*			PLAYER_ANIM_NAME_DOWN;
	static const char*			PLAYER_ANIM_NAME_LEFT;
	static const char*			PLAYER_ANIM_NAME_RIGHT;
	static const char*			PLAYER_HUD_BG;
	static double				PLAYER_MOVERATE;
	static float				PLAYER_JUJU1FILLRATE;
	static float				PLAYER_JUJU2FILLRATE;
	static float				PLAYER_JUJU3FILLRATE;
	static float				PLAYER_JUJU4FILLRATE;
	static float				PLAYER_JUJU5FILLRATE;
	static float				PLAYER_JUJU6FILLRATE;

	static const char*			PLAYER_SPELL1;	
	static const char*			PLAYER_SPELL2;	
	static const char*			PLAYER_SPELL3;	
	static const char*			PLAYER_SPELL4;	
	static const char*			PLAYER_SPELL5;	
	static const char*			PLAYER_SPELL6;

	//-------------------------------------------------------------------------------------------------
	// Public methods
	//-------------------------------------------------------------------------------------------------

	/// from Player.cpp
	Player(); // constructor
	~Player(); // destructor
	void Update();									// Update the player object and do any thinking 
	void AdjustHealth( float healthDelta );
	void AdjustMana( float delta );
	void CheckForPlayerDeath();
	void CheckForPlayerWin();
	void OnPlayerCommandKey( const byte_t key );	// Move the player

	/// from Player_Actions.cpp
	void OnPlayerUse();
	void PlantBomb();
	void UseStunner();
	void DrinkHealthPotion();
	void DrinkManaPotion();
	void DrinkInvisibilityPotion();

	/// from Player_Draw.cpp
	void Draw();									// Draw the player object
	void DrawHud();									
	void PlayerDrawInit();
	void DrawPain();
	void DrawInventory();
	void DrawActiveInventoryItemBraces( const Vector2& center, const Vector2& size, float borderThickness, bool isDisabled );
	void DrawInventoryActiveItem( int itemNumber, int itemCount );

	/// from Player_Magic.cpp
	void UpdateMagic();								// Update spells and spell effects 
	void CastMagic(int Spell);						// Cast a Spell
	void Teleport( const char* materialName, int delta );
	bool IsInvisible();

	/// from Player_Movement.cpp
	void UpdatePlayerMovement();					// Move the player, do collision checks
	void OnPlayerMoveToNewTile( const IntVector2& oldTilePos, const IntVector2& newTilePos );
	IntVector2 CalcPlayerTileCoords();
//	int CalcPlayerSemiCardinalDirection();
//	int CalcPlayerCardinalDirection();
	IntVector2 Player::CalcTileCoordsInFrontOfPlayer();
	bool ToggleGodMode( void );
	bool ToggleNoClipMode( void );

	//-------------------------------------------------------------------------------------------------
	// Public members
	//-------------------------------------------------------------------------------------------------
	Animation*			anim_idle;		// idle animation set for player
	Animation*			anim_up;		// walk up animation set for player
	Animation*			anim_down;		// walk down animation set for player
	Animation*			anim_left;		// walk left animation set for player
	Animation*			anim_right;		// walk right animation set for player
	Texture*			texHudBG;		// walk right animation set for player

	// Player attributes
	Vector2				pos;				// position of player
	float				rotDeg;				// rotation of player sprite in degrees
	Vector2				scale;				// scale of player sprite
	int					move_left;			// flag to mark if we are moving left
	int					move_right;			// flag to mark if we are moving right
	int					move_up;			// flag to mark if we are moving up
	int					move_down;			// flag to mark if we are moving down
	float				speedMultiplier;	// player's speed is scaled up or down
	float				juju1Multiplier;	// scale the speed of the mana for this spell
	float				juju2Multiplier;	// scale the speed of the mana for this spell
	float				juju3Multiplier;	// scale the speed of the mana for this spell
	float				juju4Multiplier;	// scale the speed of the mana for this spell
	float				juju5Multiplier;	// scale the speed of the mana for this spell
	float				juju6Multiplier;	// scale the speed of the mana for this spell
	bool				m_godMode ;			// true if in god mode (no damage, plus noclip)
	bool				m_noclipMode ;		// true if in noclip mode
	float				m_difficulty ;		// difficulty level. 1 is default. all damage from monsters is multiplied by this
	AABB2				m_worldBounds;


//	int					m_itemCount[ NUM_INVENTORY_ITEM_SLOTS ];
//	Texture*			m_itemTextures[ NUM_INVENTORY_ITEM_SLOTS ];
	Inventory			m_inventory;

	// Hud attributes
	float				m_painLevel;
	bool				m_wasHurtThisFrame;
	double				m_invisibilitySeconds;
	float				m_mana;
	float				health;			// player health
	float				juju1;			// mana 1
	float				juju2;			// mana 2
	float				juju3;			// mana 3
	float				juju4;			// mana 4
	float				juju5;			// mana 5
	float				juju6;			// mana 6

	// Spell Attributes
	

};


#endif // _include_Player_

