//-------------------------------------------------------------------------------------------------
// PlayerActions.cpp (game)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
// 
// This source may not be used, modified, or distributed
//  without the express written permission of The Universe.
// Damn it feels good to be a gangsta.
//-------------------------------------------------------------------------------------------------
#include "pch.hpp"
#include "TheGame.hpp"
#include "Player.hpp"


const float HEALTH_POTION_INCREASE_AMOUNT = 25.0f;
const float MANA_POTION_INCREASE_AMOUNT = 25.0f;


//-------------------------------------------------------------------------------------------------
// OnPlayerUse
//-------------------------------------------------------------------------------------------------
void Player::OnPlayerUse()
{
	/// Calculate which tile grid coordinates we're in currently
	IntVector2 playerTile = CalcPlayerTileCoords();
	IntVector2 useTile = CalcTileCoordsInFrontOfPlayer();
	theConsole.Printf( "OnPlayerUse: using tile %d,%d from tile %d,%d\n", useTile.x, useTile.y, playerTile.x, playerTile.y );

	/// Get the material type for the used tile position
	MapData& mapData = *theGame->m_currentMap;
	Material& material = mapData.GetMaterialForMapXY( useTile.x, useTile.y );
	ScriptEventHandler* onPlayerUse = material.GetEventHandlerForEvent( "OnPlayerUse" );
	if( onPlayerUse )
	{
		ExecuteScriptActions( useTile, onPlayerUse->m_scriptActions );
	}

}


//-------------------------------------------------------------------------------------------------
// PlantBomb
//-------------------------------------------------------------------------------------------------
void Player::PlantBomb()
{
	if( m_inventory.m_numBomb <= 0 )
	{
		return;
	}

	m_inventory.m_numBomb --;
	Entity::SpawnNewEntity( ENT_BOMB, pos );
}


//-------------------------------------------------------------------------------------------------
// UseStunner
//-------------------------------------------------------------------------------------------------
void Player::UseStunner()
{
	if( m_inventory.m_numStunner <= 0 )
	{
		return;
	}

	m_inventory.m_numStunner --;
	Entity::SpawnNewEntity( ENT_STUNNER, pos );
}


//-------------------------------------------------------------------------------------------------
// DrinkHealthPotion
//-------------------------------------------------------------------------------------------------
void Player::DrinkHealthPotion()
{
	if( m_inventory.m_numHealth <= 0 )
	{
		return;
	}

	/// Don't use health potions if already full
//	if( health >= 100.0f )
//	{
//		return;
//	}

	AdjustHealth( HEALTH_POTION_INCREASE_AMOUNT + ( theGame->m_player->m_difficulty * 0.5 ) );
	m_inventory.m_numHealth --;
}


//-------------------------------------------------------------------------------------------------
// DrinkManaPotion
//-------------------------------------------------------------------------------------------------
void Player::DrinkManaPotion()
{
	if( m_inventory.m_numMana <= 0 )
	{
		return;
	}

	/// Don't use mana potions if already full
//	if( m_mana >= 100.0f )
//	{
//		return;
//	}

	AdjustMana( MANA_POTION_INCREASE_AMOUNT );
	m_inventory.m_numMana --;
}


//-------------------------------------------------------------------------------------------------
// DrinkInvisibilityPotion
//-------------------------------------------------------------------------------------------------
void Player::DrinkInvisibilityPotion()
{
	if( m_inventory.m_numInvisibility <= 0 )
	{
		return;
	}

	m_invisibilitySeconds += 10.0;
	m_inventory.m_numInvisibility --;
}


