//-------------------------------------------------------------------------------------------------
// PlayerDraw.cpp (game)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
// 
// This source may not be used, modified, or distributed
//  without the express written permission of The Universe.
// Damn it feels good to be a gangsta.
//-------------------------------------------------------------------------------------------------
#include "pch.hpp"
#include "TheGame.hpp"
#include "Player.hpp"


Texture* texGeneric = NULL;		// Generic white texture used for all fill rects.


//-------------------------------------------------------------------------------------------------
// Class constants
//-------------------------------------------------------------------------------------------------
STATIC const char*	Player::PLAYER_ANIM_NAME_IDLE			= "data/Player/walkF.tga";
STATIC const char*	Player::PLAYER_ANIM_NAME_UP				= "data/Player/walkB.tga";
STATIC const char*	Player::PLAYER_ANIM_NAME_DOWN			= "data/Player/walkF.tga";
STATIC const char*	Player::PLAYER_ANIM_NAME_LEFT			= "data/Player/walkL.tga";
STATIC const char*	Player::PLAYER_ANIM_NAME_RIGHT			= "data/Player/walkR.tga";
STATIC const char*	Player::PLAYER_HUD_BG					= "data/GFX/Hud/UIback1.tga";
const char* INVENTORY_ITEM_ICON_UNKNOWN = "data/GFX/Hud/InventoryItemIcon_Unknown.tga";
const char* INVENTORY_ITEM_ICON_KEY = "data/GFX/Hud/InventoryItemIcon_Key.tga";
//const char* INVENTORY_ITEM_ICON_BOMB = "data/GFX/Hud/InventoryItemIcon_Bomb.tga";
Texture* inventory_item_icon_key = NULL;
Texture* inventory_active_item_icons[ NUM_ACTIVE_INVENTORY_ITEMS ];
Texture* inventory_icon_orb[ NUM_ORBS + 1 ];


//-------------------------------------------------------------------------------------------------
// Hud item positioning
//-------------------------------------------------------------------------------------------------
const float ITEM_LEFT_COLUMN_X = 70.0f;
const float ITEM_RIGHT_COLUMN_X = 256.0f - 75.0f;
const float ITEM_SPACING = ITEM_RIGHT_COLUMN_X - ITEM_LEFT_COLUMN_X;
const float ITEM_ROW1_Y = 768.0f - ITEM_LEFT_COLUMN_X;
const float ITEM_ROW2_Y = ITEM_ROW1_Y - 100.0f;
const float ITEM_ROW3_Y = ITEM_ROW2_Y - 100.0f;

const Vector2 itemLocation1( ITEM_LEFT_COLUMN_X, ITEM_ROW1_Y );
const Vector2 itemLocation2( ITEM_RIGHT_COLUMN_X, ITEM_ROW1_Y );
const Vector2 itemLocation3( ITEM_LEFT_COLUMN_X, ITEM_ROW2_Y );
const Vector2 itemLocation4( ITEM_RIGHT_COLUMN_X, ITEM_ROW2_Y );
const Vector2 itemLocation5( ITEM_LEFT_COLUMN_X, ITEM_ROW3_Y );
const Vector2 itemLocation6( ITEM_RIGHT_COLUMN_X, ITEM_ROW3_Y );

const Vector2 itemRectSize( 64.0f, 64.0f );
const Vector2 orbRectSize( 64.0f, 64.0f );
const float itemBorderThickness = 6.0f;

const Vector2 activeItemBasePos( 256.0f - 70.0f, 768.0f - 70.0f );
const Vector2 activeItemOffset( 0.0f, -100.0f );
Vector2 activeItemLocations[ NUM_ACTIVE_INVENTORY_ITEMS ];




//-------------------------------------------------------------------------------------------------
// DrawHud
//-------------------------------------------------------------------------------------------------
void Player::DrawHud()
{
	// Preset Hud positions
	float startx	= 28;
	float starty	= 50;

	// Draw HUD

	//theRenderer.DrawTexturedRect( *texGeneric, Vector2::ZERO, Vector2(1024,768), Vector2::ZERO, Vector2::ONE, 0.0, Vector2::ONE, Rgba(200,200,200,255) );

	//theRenderer.DrawTexturedRect( *texGeneric, Vector2::ZERO, Vector2(256,768), Vector2::ZERO, Vector2::ONE, 0.0, Vector2::ONE, Rgba(255,255,255,255) );
	//theRenderer.DrawTexturedRect( *texGeneric, Vector2(2,2), Vector2(254,766), Vector2::ZERO, Vector2::ONE, 0.0, Vector2::ONE, Rgba(20,20,20,255) );
	theRenderer.DrawTexturedRect( *texHudBG, Vector2::ZERO, Vector2(256,768), Vector2::ZERO, Vector2(1,0.75f), 0.0, Vector2::ONE, Rgba(255,255,255,255) );
	
	glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );

	// Draw Health
	theRenderer.DrawTexturedRect( *texGeneric, Vector2( (startx - 2),(starty - 2) ), Vector2( (startx + 202),(starty + 20)), Vector2::ZERO, Vector2::ONE, 0.0, Vector2::ONE, Rgba(255,255,255,255) );
	theRenderer.DrawTexturedRect( *texGeneric, Vector2(startx,starty), Vector2( (startx + 200), (starty + 18) ), Vector2::ZERO, Vector2::ONE, 0.0, Vector2::ONE, Rgba(0,0,0,255) );
	theRenderer.DrawTexturedRect( *texGeneric, Vector2(startx,starty), Vector2( startx + health*2,(starty + 18) ), Vector2::ZERO, Vector2::ONE, 0.0, Vector2::ONE, Rgba(255,0,0,255) );
	theRenderer.XYPrintf( startx + 145.0f, starty - 10, "Health");

	// Draw Mana
	starty += 100;
	theRenderer.DrawTexturedRect( *texGeneric, Vector2( (startx - 2),(starty - 2) ), Vector2( (startx + 202),(starty + 20)), Vector2::ZERO, Vector2::ONE, 0.0, Vector2::ONE, Rgba(255,255,255,255) );
	theRenderer.DrawTexturedRect( *texGeneric, Vector2(startx,starty), Vector2( (startx + 200), (starty + 18) ), Vector2::ZERO, Vector2::ONE, 0.0, Vector2::ONE, Rgba(0,0,0,255) );
	theRenderer.DrawTexturedRect( *texGeneric, Vector2(startx,starty), Vector2( startx + m_mana*2,(starty + 18) ), Vector2::ZERO, Vector2::ONE, 0.0, Vector2::ONE, Rgba(0,0,255,255) );
	theRenderer.XYPrintf( startx + 155.0f, starty - 10, "Mana");

/*
	starty += 100;
	// Draw Juju1
	theRenderer.XYPrintf( startx, starty - 10, "Spell 1");
	theRenderer.DrawTexturedRect( *texGeneric, Vector2( (startx - 2),(starty - 2) ), Vector2( (startx + 202),(starty + 20)), Vector2::ZERO, Vector2::ONE, 0.0, Vector2::ONE, Rgba(255,255,255,255) );
	theRenderer.DrawTexturedRect( *texGeneric, Vector2(startx,starty), Vector2( (startx + 200), (starty + 18) ), Vector2::ZERO, Vector2::ONE, 0.0, Vector2::ONE, Rgba(0,0,0,255) );
	if (juju1 == 200)
		theRenderer.DrawTexturedRect( *texGeneric, Vector2(startx,starty), Vector2( startx + juju1,(starty + 18) ), Vector2::ZERO, Vector2::ONE, 0.0, Vector2::ONE, Rgba(0,255,0,255) );
	else
		theRenderer.DrawTexturedRect( *texGeneric, Vector2(startx,starty), Vector2( startx + juju1,(starty + 18) ), Vector2::ZERO, Vector2::ONE, 0.0, Vector2::ONE, Rgba(0,55,0,255) );

	starty += 50;
	// Draw Juju2
	theRenderer.XYPrintf( startx, starty - 10, "Spell 2");
	theRenderer.DrawTexturedRect( *texGeneric, Vector2( (startx - 2),(starty - 2) ), Vector2( (startx + 202),(starty + 20)), Vector2::ZERO, Vector2::ONE, 0.0, Vector2::ONE, Rgba(255,255,255,255) );
	theRenderer.DrawTexturedRect( *texGeneric, Vector2(startx,starty), Vector2( (startx + 200), (starty + 18) ), Vector2::ZERO, Vector2::ONE, 0.0, Vector2::ONE, Rgba(0,0,0,255) );
	if (juju2 == 200)
		theRenderer.DrawTexturedRect( *texGeneric, Vector2(startx,starty), Vector2( startx + juju2,(starty + 18) ), Vector2::ZERO, Vector2::ONE, 0.0, Vector2::ONE, Rgba(0,0,255,255) );
	else
		theRenderer.DrawTexturedRect( *texGeneric, Vector2(startx,starty), Vector2( startx + juju2,(starty + 18) ), Vector2::ZERO, Vector2::ONE, 0.0, Vector2::ONE, Rgba(0,0,55,255) );
	
	starty += 50;
	// Draw Juju3
	theRenderer.XYPrintf( startx, starty - 10, "Spell 3");
	theRenderer.DrawTexturedRect( *texGeneric, Vector2( (startx - 2),(starty - 2) ), Vector2( (startx + 202),(starty + 20)), Vector2::ZERO, Vector2::ONE, 0.0, Vector2::ONE, Rgba(255,255,255,255) );
	theRenderer.DrawTexturedRect( *texGeneric, Vector2(startx,starty), Vector2( (startx + 200), (starty + 18) ), Vector2::ZERO, Vector2::ONE, 0.0, Vector2::ONE, Rgba(0,0,0,255) );
	if (juju3 == 200)
		theRenderer.DrawTexturedRect( *texGeneric, Vector2(startx,starty), Vector2( startx + juju3,(starty + 18) ), Vector2::ZERO, Vector2::ONE, 0.0, Vector2::ONE, Rgba(255,255,0,255) );
	else
		theRenderer.DrawTexturedRect( *texGeneric, Vector2(startx,starty), Vector2( startx + juju3,(starty + 18) ), Vector2::ZERO, Vector2::ONE, 0.0, Vector2::ONE, Rgba(55,55,0,255) );

	starty += 50;
	// Draw Juju4
	theRenderer.XYPrintf( startx, starty - 10, "Spell 4");
	theRenderer.DrawTexturedRect( *texGeneric, Vector2( (startx - 2),(starty - 2) ), Vector2( (startx + 202),(starty + 20)), Vector2::ZERO, Vector2::ONE, 0.0, Vector2::ONE, Rgba(255,255,255,255) );
	theRenderer.DrawTexturedRect( *texGeneric, Vector2(startx,starty), Vector2( (startx + 200), (starty + 18) ), Vector2::ZERO, Vector2::ONE, 0.0, Vector2::ONE, Rgba(0,0,0,255) );
	if (juju4 == 200)
		theRenderer.DrawTexturedRect( *texGeneric, Vector2(startx,starty), Vector2( startx + juju4,(starty + 18) ), Vector2::ZERO, Vector2::ONE, 0.0, Vector2::ONE, Rgba(0,255,255,255) );
	else
		theRenderer.DrawTexturedRect( *texGeneric, Vector2(startx,starty), Vector2( startx + juju4,(starty + 18) ), Vector2::ZERO, Vector2::ONE, 0.0, Vector2::ONE, Rgba(0,55,55,255) );

	starty += 50;
	// Draw Juju5
	theRenderer.XYPrintf( startx, starty - 10, "Spell 5");
	theRenderer.DrawTexturedRect( *texGeneric, Vector2( (startx - 2),(starty - 2) ), Vector2( (startx + 202),(starty + 20)), Vector2::ZERO, Vector2::ONE, 0.0, Vector2::ONE, Rgba(255,255,255,255) );
	theRenderer.DrawTexturedRect( *texGeneric, Vector2(startx,starty), Vector2( (startx + 200), (starty + 18) ), Vector2::ZERO, Vector2::ONE, 0.0, Vector2::ONE, Rgba(0,0,0,255) );
	if (juju5 == 200)
		theRenderer.DrawTexturedRect( *texGeneric, Vector2(startx,starty), Vector2( startx + juju5,(starty + 18) ), Vector2::ZERO, Vector2::ONE, 0.0, Vector2::ONE, Rgba(255,0,255,255) );
	else
		theRenderer.DrawTexturedRect( *texGeneric, Vector2(startx,starty), Vector2( startx + juju5,(starty + 18) ), Vector2::ZERO, Vector2::ONE, 0.0, Vector2::ONE, Rgba(55,0,55,255) );

	starty += 50;
	// Draw Juju6
	theRenderer.XYPrintf( startx, starty - 10, "Spell 6");
	theRenderer.DrawTexturedRect( *texGeneric, Vector2( (startx - 2),(starty - 2) ), Vector2( (startx + 202),(starty + 20)), Vector2::ZERO, Vector2::ONE, 0.0, Vector2::ONE, Rgba(255,255,255,255) );
	theRenderer.DrawTexturedRect( *texGeneric, Vector2(startx,starty), Vector2( (startx + 200), (starty + 18) ), Vector2::ZERO, Vector2::ONE, 0.0, Vector2::ONE, Rgba(0,0,0,255) );
	if (juju6 == 200)
		theRenderer.DrawTexturedRect( *texGeneric, Vector2(startx,starty), Vector2( startx + juju6,(starty + 18) ), Vector2::ZERO, Vector2::ONE, 0.0, Vector2::ONE, Rgba(240,100,30,255) );
	else
		theRenderer.DrawTexturedRect( *texGeneric, Vector2(startx,starty), Vector2( startx + juju6,(starty + 18) ), Vector2::ZERO, Vector2::ONE, 0.0, Vector2::ONE, Rgba(50,10,0,255) );


	starty += 60;
	theRenderer.XYPrintf( startx, starty - 10, "p:%0.3f %0.3f", pos.x, pos.y );
	starty += 25;
	theRenderer.XYPrintf( startx, starty - 10, "f:%0.2f",
		theGame->m_lights.m_mapRadiusScale * theGame->m_lights.m_playerRadiusScale );
	starty += 25;
	theRenderer.XYPrintf( startx, starty - 10, "(mf:%0.2f pf:%0.2f)", 
		theGame->m_lights.m_mapRadiusScale, theGame->m_lights.m_playerRadiusScale );
*/
	/// Draw Inventory Items
	DrawInventory();
}


//-------------------------------------------------------------------------------------------------
// Draw
//-------------------------------------------------------------------------------------------------
void Player::Draw()
{
	//--------------------
	// Draw Player
	//---------------------
	Vector2 p( pos * 32 );
	//anim_idle->Draw(p, scale, rotDeg);

	Rgba color = Rgba::WHITE;
	if( IsInvisible() )
	{
		color.a = 64;
	}

	if (move_up > 0 && move_left > 0)
		anim_left->Draw( p, scale, 0, color );
	else if (move_up > 0 && move_right > 0)
		anim_right->Draw( p, scale, 0, color );
	else if (move_down > 0 && move_right > 0)
		anim_right->Draw( p, scale, 0, color );
	else if (move_down > 0 && move_left > 0)
		anim_left->Draw( p, scale, 0, color );
	else if (move_up > 0)
		anim_up->Draw( p, scale, 0, color );
	else if (move_down > 0)
		anim_down->Draw( p, scale, 0, color );
	else if (move_left > 0)
		anim_left->Draw( p, scale, 0, color );
	else if (move_right > 0)
		anim_right->Draw( p, scale, 0, color );
	else
		anim_down->Draw( p, scale, 0, color );
}


//-------------------------------------------------------------------------------------------------
// DrawPain
//-------------------------------------------------------------------------------------------------
void Player::DrawPain()
{
	//-------------------------------------------------------------------------------------------------
	// Do Pain logic - FIXME: MOVE THIS SOMEWHERE ELSE!!!
	//-------------------------------------------------------------------------------------------------
	if( m_wasHurtThisFrame )
	{
		m_wasHurtThisFrame = false;
//		m_painLevel += 1.0f * (float) theApplication.GetFrameSeconds();
	}

	m_painLevel -= 0.5f * (float) theApplication.GetFrameSeconds();
	m_painLevel = ClampFloat( m_painLevel, 0.0f, 1.0f );

	//-------------------------------------------------------------------------------------------------
	// Draw Pain Effect
	//-------------------------------------------------------------------------------------------------
	glDisable( GL_TEXTURE_2D );
	glShadeModel( GL_SMOOTH );
	glColor4f( 1.0f, 0.0f, 0.0f, m_painLevel );
	glBegin( GL_QUADS );
	{
		glVertex2f( 0.0f, 0.0f );
		glVertex2f( 1024.0f, 0.0f );
		glVertex2f( 1024.0f, 768.0f );
		glVertex2f( 0.0f, 768.0f );
	}
	glEnd(); // GL_QUADS
}


//-------------------------------------------------------------------------------------------------
// PlayerDrawInit
//-------------------------------------------------------------------------------------------------
void Player::PlayerDrawInit()
{
	// Create a generic texture
	texGeneric = new Texture( "data/gfx/0_white.tga" );
	VALIDATE( texGeneric );

	// create player anims
	anim_idle = new Animation( PLAYER_ANIM_NAME_IDLE, 7 );
	VALIDATE( anim_idle );

	anim_up = new Animation( PLAYER_ANIM_NAME_UP, 7 );
	VALIDATE( anim_up );
	anim_down = new Animation( PLAYER_ANIM_NAME_DOWN, 7 );
	VALIDATE( anim_down );
	anim_left = new Animation( PLAYER_ANIM_NAME_LEFT, 7 );
	VALIDATE( anim_left );
	anim_right = new Animation( PLAYER_ANIM_NAME_RIGHT, 7 );
	VALIDATE( anim_right );

	// Create Hud BG
	texHudBG = new Texture( PLAYER_HUD_BG );
//	m_itemTextures[ 0 ] = new Texture( INVENTORY_ITEM_ICON_UNKNOWN );
//	m_itemTextures[ 1 ] = new Texture( INVENTORY_ITEM_ICON_UNKNOWN );
//	m_itemTextures[ 2 ] = new Texture( INVENTORY_ITEM_ICON_UNKNOWN );
//	m_itemTextures[ 3 ] = new Texture( INVENTORY_ITEM_ICON_UNKNOWN );
//	m_itemTextures[ 4 ] = new Texture( INVENTORY_ITEM_ICON_UNKNOWN );
//	m_itemTextures[ 5 ] = new Texture( INVENTORY_ITEM_ICON_UNKNOWN );

	VALIDATE( texHudBG );

	// hud info
	health = 100;	
	m_mana = 100.0f;
	juju1 = 100;
	juju2 = 50;	
	juju3 = 0;	
	juju4 = 0;	
	juju5 = 0;	
	juju6 = 0;
	juju1Multiplier = 1;
	juju2Multiplier = 1;
	juju3Multiplier = 1;
	juju4Multiplier = 1;
	juju5Multiplier = 1;
	juju6Multiplier = 1;

	/// Load inventory gfx
	inventory_item_icon_key = new Texture( INVENTORY_ITEM_ICON_KEY );
	inventory_active_item_icons[ ACTIVE_INVENTORY_ITEM_BOMB ]			= new Texture( "data/GFX/Hud/InventoryItemIcon_Bomb.tga" );
	inventory_active_item_icons[ ACTIVE_INVENTORY_ITEM_STUNNER ]		= new Texture( "data/GFX/Hud/InventoryItemIcon_StunBomb.tga" );
	inventory_active_item_icons[ ACTIVE_INVENTORY_ITEM_HEALTH ]			= new Texture( "data/GFX/Hud/InventoryItemIcon_Health.tga" );
	inventory_active_item_icons[ ACTIVE_INVENTORY_ITEM_MANA ]			= new Texture( "data/GFX/Hud/InventoryItemIcon_Mana.tga" );
	inventory_active_item_icons[ ACTIVE_INVENTORY_ITEM_INVISIBILITY ]	= new Texture( "data/GFX/Hud/InventoryItemIcon_Invisibility.tga" );

	/// Calculate inventory gfx coordinates
	for( int itemNum = 0; itemNum < NUM_ACTIVE_INVENTORY_ITEMS; itemNum ++ )
	{
		activeItemLocations[ itemNum ] = activeItemBasePos + (activeItemOffset * (float) itemNum);
	}

	for( int o = 0; o < NUM_ORBS + 1; o ++ )
	{
		char fileName[ 1024 ];
		sprintf( fileName, "data/GFX/Hud/InventoryItemIcon_Goal%d.tga", o );
		inventory_icon_orb[ o ] = new Texture( fileName );
	}
}



//-------------------------------------------------------------------------------------------------
// DrawActiveInventoryItemBraces
//-------------------------------------------------------------------------------------------------
void Player::DrawActiveInventoryItemBraces( const Vector2& center, const Vector2& size, float borderThickness, bool isDisabled )
{
	const float braceThickness = 6.0f;
	const float braceLipSize = 8.0f;

	Vector2 border( borderThickness, borderThickness );
	Vector2 braceBorder( braceThickness, braceThickness );
	Vector2 halfInnerRectSize = itemRectSize * 0.5f;
	Vector2 innerRectMins = center - halfInnerRectSize;
	Vector2 innerRectMaxs = center + halfInnerRectSize;
	Vector2 outerRectMins = innerRectMins - border;
	Vector2 outerRectMaxs = innerRectMaxs + border;
	Vector2 braceRectMins = outerRectMins - braceBorder;
	Vector2 braceRectMaxs = outerRectMaxs + braceBorder;
	Vector2 braceLipsMins = innerRectMins + Vector2( braceLipSize, braceLipSize );
	Vector2 braceLipsMaxs = innerRectMaxs - Vector2( braceLipSize, braceLipSize );
//	float braceRectLeftStopX = innerRectMins.x;
//	float braceRectRightStopX = innerRectMaxs.x;

	glDisable( GL_TEXTURE_2D );

	/// Draw selection highlight
	glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
	glColor4f( 1.0f, 1.0f, 0.0f, 1.0f );
	glBegin( GL_QUADS );
	{
		glVertex2f( braceRectMins.x, braceRectMins.y );
		glVertex2f( braceLipsMins.x, braceRectMins.y );
		glVertex2f( braceLipsMins.x, outerRectMins.y );
		glVertex2f( outerRectMins.x, outerRectMins.y );

		glVertex2f( braceRectMins.x, braceRectMaxs.y );
		glVertex2f( braceLipsMins.x, braceRectMaxs.y );
		glVertex2f( braceLipsMins.x, outerRectMaxs.y );
		glVertex2f( outerRectMins.x, outerRectMaxs.y );

		glVertex2f( braceRectMaxs.x, braceRectMins.y );
		glVertex2f( braceLipsMaxs.x, braceRectMins.y );
		glVertex2f( braceLipsMaxs.x, outerRectMins.y );
		glVertex2f( outerRectMaxs.x, outerRectMins.y );

		glVertex2f( braceRectMaxs.x, braceRectMaxs.y );
		glVertex2f( braceLipsMaxs.x, braceRectMaxs.y );
		glVertex2f( braceLipsMaxs.x, outerRectMaxs.y );
		glVertex2f( outerRectMaxs.x, outerRectMaxs.y );

		glVertex2f( braceRectMins.x, braceRectMins.y );
		glVertex2f( outerRectMins.x, outerRectMins.y );
		glVertex2f( outerRectMins.x, outerRectMaxs.y );
		glVertex2f( braceRectMins.x, braceRectMaxs.y );

		glVertex2f( braceRectMaxs.x, braceRectMins.y );
		glVertex2f( outerRectMaxs.x, outerRectMins.y );
		glVertex2f( outerRectMaxs.x, outerRectMaxs.y );
		glVertex2f( braceRectMaxs.x, braceRectMaxs.y );
	}
	glEnd(); // GL_QUADS
}



//-------------------------------------------------------------------------------------------------
// DrawInventory
//-------------------------------------------------------------------------------------------------
void Player::DrawInventory()
{
	int itemCount;
	Vector2 position;
	Vector2 offset;
	Vector2 numberOffset( 20.0f, 27.0f );
	Vector2 numberOffsetDoubleDigits( 12.0f, 27.0f );

	if( IsInvisible() )
	{
		theRenderer.XYPrintf( 100.0f, 100.0f, "Invisible (%.0f seconds remain)", (float)m_invisibilitySeconds );
	}

	/// Draw keys
	itemCount = m_inventory.m_numKey;
	theGame->DrawFrameShadedImage( inventory_item_icon_key, itemLocation1, itemRectSize, itemBorderThickness, itemCount == 0 );
	offset = itemCount > 9 ? numberOffsetDoubleDigits : numberOffset;
	theRenderer.XYPrintf( itemLocation1 + offset, "%d", itemCount );

	/// Draw orbs
	itemCount = ClampInt( m_inventory.m_numOrb, 0, 9 );
	position = itemLocation3;
	theGame->DrawFrameShadedImage( inventory_icon_orb[ itemCount ], position, orbRectSize, itemBorderThickness, false );

	/// Draw active inventory items
	DrawInventoryActiveItem( ACTIVE_INVENTORY_ITEM_BOMB,			m_inventory.m_numBomb );
	DrawInventoryActiveItem( ACTIVE_INVENTORY_ITEM_STUNNER,			m_inventory.m_numStunner );
	DrawInventoryActiveItem( ACTIVE_INVENTORY_ITEM_HEALTH,			m_inventory.m_numHealth );
	DrawInventoryActiveItem( ACTIVE_INVENTORY_ITEM_MANA,			m_inventory.m_numMana );
	DrawInventoryActiveItem( ACTIVE_INVENTORY_ITEM_INVISIBILITY,	m_inventory.m_numInvisibility );

	/// Draw active inventory item braces
	DrawActiveInventoryItemBraces( activeItemLocations[ m_inventory.m_activeInventoryItemNumber ], itemRectSize, itemBorderThickness, false );
}


//-------------------------------------------------------------------------------------------------
// DrawInventoryActiveItem
//-------------------------------------------------------------------------------------------------
void Player::DrawInventoryActiveItem( int itemNumber, int itemCount )
{
	Vector2 numberOffset( 20.0f, 27.0f );
	Vector2 numberOffsetDoubleDigits( 12.0f, 27.0f );

	Texture* texture = 	inventory_active_item_icons[ itemNumber ];
	Vector2 position = activeItemLocations[ itemNumber ];
	Vector2 offset = itemCount > 9 ? numberOffsetDoubleDigits : numberOffset;

	theGame->DrawFrameShadedImage( texture, position, itemRectSize, itemBorderThickness, itemCount == 0 );
	theRenderer.XYPrintf( position + offset, "%d", itemCount );
}



