//-------------------------------------------------------------------------------------------------
// PlayerMovement.cpp (game)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
// 
// This source may not be used, modified, or distributed
//  without the express written permission of The Universe.
// Damn it feels good to be a gansta.
//-------------------------------------------------------------------------------------------------

#include "pch.hpp"
#include "TheGame.hpp"
#include "Player.hpp"



//-------------------------------------------------------------------------------------------------
// Class constants
//-------------------------------------------------------------------------------------------------
STATIC double		Player::PLAYER_MOVERATE					= 5.0f;
const Vector2 g_playerSize( 16.0f, 20.0f );


//-------------------------------------------------------------------------------------------------
// UpdatePlayerMovement
//-------------------------------------------------------------------------------------------------
void Player::UpdatePlayerMovement()
{
	const unsigned int collisionFlags = MATFLAG_BLOCKS_PLAYERS;

	/// Determine which tile the player is "mostly" in before moving
	IntVector2 oldTilePos = CalcPlayerTileCoords();

	//--------------------------------------------------------
	// MOVE PLAYER
	//--------------------------------------------------------
	float movementThisFrame = (float)(PLAYER_MOVERATE * theApplication.GetFrameSeconds() );
	movementThisFrame *= speedMultiplier;

	Vector2 newPosition = pos;
	Vector2 oldPosition = pos;
	Vector2 storedPosition = pos;

	/// First, try to apply sidewise movement
	// LEFT
	if ( move_left > 0 )
		newPosition.x -= movementThisFrame;
	// RIGHT
	if ( move_right > 0 )
		newPosition.x += movementThisFrame;

	/// Check if new position collides with anything
	Vector2 mins = newPosition * 32.0f - (g_playerSize * 0.5f);
	Vector2 maxs = newPosition * 32.0f + (g_playerSize * 0.5f);
	AABB2 newBounds( mins, maxs );
	storedPosition.x = newPosition.x;
	if( theGame->m_noClip || !theGame->DoBoundsCollideWithMap( newBounds, collisionFlags ) )
	{
		pos = newPosition;
	}

	/// Next, try to apply up/down movement
	newPosition = pos;
	// UP
	if ( move_up > 0 )
		newPosition.y -= movementThisFrame;
	// DOWN
	if ( move_down > 0 )
		newPosition.y += movementThisFrame;

	/// Check if new position collides with anything
	mins = newPosition * 32.0f - (g_playerSize * 0.5f);
	maxs = newPosition * 32.0f + (g_playerSize * 0.5f);
	newBounds.SetFromMinsMaxs( mins, maxs );
	storedPosition.y = newPosition.y;
	if( theGame->m_noClip || !theGame->DoBoundsCollideWithMap( newBounds, collisionFlags ) )
	{
		pos = newPosition;
	}

	Vector2 delta = storedPosition - oldPosition;
	if( !delta.IsZero() )
		rotDeg = (float) delta.GetDegrees();

	/// Determine which tile the player is "mostly" in after moving
	IntVector2 newTilePos = CalcPlayerTileCoords();

	/// Determine if the player moved from one tile into another
	if( newTilePos != oldTilePos )
	{
		OnPlayerMoveToNewTile( oldTilePos, newTilePos );
	}

	Vector2 halfPlayerBounds = 0.5f * g_playerSize * (1.0f / 32.0f);
	m_worldBounds.SetFromMinsMaxs( pos - halfPlayerBounds, pos + halfPlayerBounds );
}


//-------------------------------------------------------------------------------------------------
// OnPlayerMoveToNewTile
//-------------------------------------------------------------------------------------------------
void Player::OnPlayerMoveToNewTile( const IntVector2& oldTilePos, const IntVector2& newTilePos )
{
//	theConsole.Printf( "Player has moved from tile %d,%d into tile %d,%d\n", oldTilePos.x, oldTilePos.y, newTilePos.x, newTilePos.y );

	/// Get the material type for the new tile position
	MapData& mapData = *theGame->m_currentMap;
	Material& material = mapData.GetMaterialForMapXY( newTilePos.x, newTilePos.y );
	ScriptEventHandler* onPlayerEnter = material.GetEventHandlerForEvent( "OnPlayerEnter" );
	if( onPlayerEnter )
	{
		ExecuteScriptActions( newTilePos, onPlayerEnter->m_scriptActions );
	}
}


//-------------------------------------------------------------------------------------------------
// CalcPlayerTileCoords
//-------------------------------------------------------------------------------------------------
IntVector2 Player::CalcPlayerTileCoords()
{
	return IntVector2( (int)pos.x, (int)pos.y );
}



//-------------------------------------------------------------------------------------------------
// CalcTileCoordsInFrontOfPlayer
//-------------------------------------------------------------------------------------------------
IntVector2 Player::CalcTileCoordsInFrontOfPlayer()
{
	Vector2 forwardVector;
	forwardVector.SetFromRadiusCosSinDegrees( 1.0f, rotDeg );
	Vector2 forwardPos = pos + forwardVector;

	return IntVector2( (int)forwardPos.x, (int)forwardPos.y );
}


