//-------------------------------------------------------------------------------------------------
// TheGame.hpp (game)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
// 
// This source may not be used, modified, or distributed
//  without the express written permission of Brian 'Squirrel' Eiserloh.
//-------------------------------------------------------------------------------------------------
#ifndef _include_TheGame_
#define _include_TheGame_


//-------------------------------------------------------------------------------------------------
// Included header files
//-------------------------------------------------------------------------------------------------
#include "gameCommon.hpp"
#include "Animation.hpp"
#include "MapDisplay.hpp"
#include "EntityDisplay.hpp"
#include "MapData.hpp"
#include "Player.hpp"
#include "Lights.hpp"


//-------------------------------------------------------------------------------------------------
// GameState
//-------------------------------------------------------------------------------------------------
enum GameState
{
	GAMESTATE_ATTRACT,
	GAMESTATE_PLAYING,
	GAMESTATE_GAMEOVER,
	GAMESTATE_YOUWIN,
	GAMESTATE_EDITMAP,
};


///////////////////////////////////////////////////////////////////////////////////////////////////
//
//		TheGame
// 
///////////////////////////////////////////////////////////////////////////////////////////////////
class TheGame
{
private:
public:
	/// Constructors / destructors
	TheGame(); // default constructor
	~TheGame(); // destructor

	/// from TheGame.cpp
	void PreInit();
	void PostInit();
	void RunFrame();
	void RunFrame_Playing();
	void RunFrame_Attract();
	void RunFrame_GameOver();
	void RunFrame_YouWin();
	void RunFrame_EditMap();
	void ChangeState( GameState newState );
	bool WarpToLevel( unsigned int newMapNumber, int newTileX = -1, int newTileY = -1 );
	bool WarpToLevelSet( const char* mapSet, unsigned int newMapNumber=1, int newTileX = -1, int newTileY = -1 );

	/// from TheGame_Update.cpp
	void UpdateEverything();

	/// from TheGame_Draw.cpp
	void SetUpWorldCoordinateSystem();
	void SetUpScreenCoordinateSystem();
	void DrawEverything();
	void DrawFrameShading( const Vector2& center, const Vector2& innerRectSize, float borderThickness );
	void DrawFrameShadedImage( const Texture* texture, const Vector2& center, const Vector2& innerRectSize, float borderThickness, bool isGreyedOut );

	/// from TheGame_Input.cpp
	void OnKeyDown( const byte_t key );
	void OnKeyUp( const byte_t key );

	/// from TheGame_Physics.cpp
	bool DoBoundsCollideWithMap( const AABB2& bounds, unsigned int collisionTypeFlags );
	bool IsPointWithinGivenRadiusOfPlayer( const Vector2& point, float radius );

	bool ExecuteCommand();

	Player*		m_player;			// The player
	bool		m_noClip;
	int			m_currentMapNumber;
	MapData*	m_currentMap;
	GameState	m_gameState;

	Texture*	m_attractScreenTexture;
	Texture*	m_gameOverScreenTexture;
	Texture*	m_youWinScreenTexture;

	Lights		m_lights;


protected:

	void _InitMap();

//	Map						_map;
	MapDisplay				_mapDisplay;
	EntityDisplay			_entityDisplay;
//	std::vector<MapData*>	_mapData;

	bool					_commandConsoleEnabled ;
	char					_commandBuffer[ 255 ] ;
	int						_commandBufferPosition ;
};


//-------------------------------------------------------------------------------------------------
// Externally viewable globals
//-------------------------------------------------------------------------------------------------
extern TheGame* theGame;


#endif // _include_TheGame_
