//-------------------------------------------------------------------------------------------------
// TheGameInterface.hpp (game)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
// 
// Header file for TheGameInterface, an interface class derived from
//	the engine's TheGameInterfaceAbstract.  This (singleton) object
//	instantiates itself statically at file scope, before entering
//	WinMain.
// 
// TheGameInterface serves as the primary method for the engine to
//	call into the game.  See Engine/TheGameInterfaceAbstract for details.
//-------------------------------------------------------------------------------------------------
#ifndef _include_TheGameInterface_
#define _include_TheGameInterface_


//-------------------------------------------------------------------------------------------------
// Included header files
//-------------------------------------------------------------------------------------------------
#include "gameCommon.hpp"


///////////////////////////////////////////////////////////////////////////////////////////////////
//
//		TheGameInterface
// 
///////////////////////////////////////////////////////////////////////////////////////////////////
class TheGameInterface : public TheGameInterfaceAbstract
{
public:
	/// Constructors / destructors
	TheGameInterface(); // default constructor
	~TheGameInterface(); // destructor

	/// Virtual methods inherited by TheGameAbstract that may be implemented by the game
	VIRTUAL void PreInit();	// called before subsystem initialization
	VIRTUAL void PostInit();	// called after subsystem initialization
	VIRTUAL void RunFrame();	// called every frame, after input but before rendering
	VIRTUAL void Shutdown();	// called on application shutdown
	VIRTUAL void ProcessCommandLine( std::string cmdLine );
	VIRTUAL const bool ProcessInputEvent( const TheInputSystem::inputEventType_e eventType, const byte_t key );
};



//-------------------------------------------------------------------------------------------------
// Global singleton instance
//-------------------------------------------------------------------------------------------------
extern TheGameInterface theGameInterface;


#endif // _include_TheGameInterface_
