//-------------------------------------------------------------------------------------------------
// TheGame_Draw.cpp (game)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
// 
// This source may not be used, modified, or distributed
//  without the express written permission of Brian 'Squirrel' Eiserloh.
//-------------------------------------------------------------------------------------------------
#include "pch.hpp"
#include "TheGame.hpp"
#include "Particle.hpp"


//-------------------------------------------------------------------------------------------------
// SetUpCoordinateSystem
// 
// Set up the drawing context
//-------------------------------------------------------------------------------------------------
void TheGame::SetUpScreenCoordinateSystem()
{
	/// Get the size and center of the current video mode
	Vector2 bottomRight( (float) theVideoSystem.GetCurrentVidModeWidth(), (float) theVideoSystem.GetCurrentVidModeHeight() );
	Vector2 topLeft( 0, 0 );

	/// Set up a 2d drawing coordinate system with 0.0 at the bottom-left
	theRenderer.SetCoordinateSystem( topLeft, bottomRight);
	glTranslatef( 0.375, 0.375, 0 );

	if( m_player->m_wasHurtThisFrame )
	{
		float screenShakeX = RandomFloat( 0.0f, m_player->m_painLevel * 30.0f );
		float screenShakeY = RandomFloat( 0.0f, m_player->m_painLevel * 30.0f );
		glTranslatef( screenShakeX, screenShakeY, 0.0f );
	}
}

//-------------------------------------------------------------------------------------------------
void TheGame::SetUpWorldCoordinateSystem()
{
	Vector2 center( 256 + 768/2, 768/2);
	theRenderer.TranslateCoordinateSystem( m_player->pos * 32, center );
	glTranslatef( 0.375, 0.375, 0 );
}


//-------------------------------------------------------------------------------------------------
// DrawEverything
// 
// Render the scene
//-------------------------------------------------------------------------------------------------
void TheGame::DrawEverything()
{
	glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );

	SetUpScreenCoordinateSystem();
	_mapDisplay.Draw( m_currentMapNumber );

	SetUpWorldCoordinateSystem();
	_entityDisplay.Draw( m_currentMapNumber );
	m_player->Draw();

	SetUpScreenCoordinateSystem();
	m_lights.draw();

	SetUpScreenCoordinateSystem();
	m_player->DrawHud();

	SetUpWorldCoordinateSystem();
	Particle::DrawAll();

	SetUpScreenCoordinateSystem();
	m_player->DrawPain();


//	int numEntities = m_currentMap->entityList.size();
//	theRenderer.XYPrintf( 260.0f, 20.0f, "%d entities active on map", numEntities );


	if ( _commandConsoleEnabled )
	{
		Rgba	backgroundColor( 0, 0, 0, 127 );
		float	centerX	= 256.0f + 768.0f/2.0f ;
		float	centerY	= 768.0f/2.0f;
		float	width	= 200.0f ;
		float	height	= 24.0f ;
		float	left	= centerX - width ;
		float	right	= centerX + width ;
		float	top		= centerY - height ;
		float	bottom	= centerY + height ;

		AABB2 bounds( left, top, right, bottom );
		theRenderer.DrawAABB2( bounds, backgroundColor, true );
		theRenderer.XYPrintf( left+20.0f, centerY+6.0f, "%s", _commandBuffer );
	}
}


//-------------------------------------------------------------------------------------------------
// DrawFrameShading
//-------------------------------------------------------------------------------------------------
void TheGame::DrawFrameShading( const Vector2& center, const Vector2& innerRectSize, float borderThickness )
{
	Vector2 border( borderThickness, borderThickness );
	Vector2 halfInnerRectSize = innerRectSize * 0.5f;
	Vector2 innerRectMins = center - halfInnerRectSize;
	Vector2 innerRectMaxs = center + halfInnerRectSize;
	Vector2 outerRectMins = innerRectMins - border;
	Vector2 outerRectMaxs = innerRectMaxs + border;

	glDisable( GL_TEXTURE_2D );

	/// Draw additive highlights
	glBlendFunc( GL_SRC_ALPHA, GL_ONE );
	glColor4f( 1.0f, 1.0f, 1.0f, 0.15f );
	glBegin( GL_QUADS );
	{
		glVertex2f( outerRectMins.x, outerRectMins.y );
		glVertex2f( innerRectMins.x, innerRectMins.y );
		glVertex2f( innerRectMins.x, innerRectMaxs.y );
		glVertex2f( outerRectMins.x, outerRectMaxs.y );

		glVertex2f( outerRectMins.x, outerRectMaxs.y );
		glVertex2f( innerRectMins.x, innerRectMaxs.y );
		glVertex2f( innerRectMaxs.x, innerRectMaxs.y );
		glVertex2f( outerRectMaxs.x, outerRectMaxs.y );
	}
	glEnd(); // GL_QUADS

	/// Draw multiplicative lowlights
	glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
	glColor4f( 0.0f, 0.0f, 0.0f, 0.4f );
	glBegin( GL_QUADS );
	{
		glVertex2f( innerRectMins.x, innerRectMins.y );
		glVertex2f( outerRectMins.x, outerRectMins.y );
		glVertex2f( outerRectMaxs.x, outerRectMins.y );
		glVertex2f( innerRectMaxs.x, innerRectMins.y );

		glVertex2f( outerRectMaxs.x, outerRectMins.y );
		glVertex2f( innerRectMaxs.x, innerRectMins.y );
		glVertex2f( innerRectMaxs.x, innerRectMaxs.y );
		glVertex2f( outerRectMaxs.x, outerRectMaxs.y );

		glColor4f( 0.0f, 0.0f, 0.0f, 0.2f );
		glVertex2f( innerRectMins.x, innerRectMins.y );
		glVertex2f( innerRectMaxs.x, innerRectMins.y );
		glVertex2f( innerRectMaxs.x, innerRectMaxs.y );
		glVertex2f( innerRectMins.x, innerRectMaxs.y );
	}
	glEnd(); // GL_QUADS
}


//-------------------------------------------------------------------------------------------------
// DrawFrameShading
//-------------------------------------------------------------------------------------------------
void TheGame::DrawFrameShadedImage( const Texture* texture, const Vector2& center, const Vector2& innerRectSize, float borderThickness, bool isGreyedOut )
{
	DrawFrameShading( center, innerRectSize, borderThickness );
	if( texture )
	{
		Rgba color = Rgba::WHITE;
		if( isGreyedOut )
		{
			color = Rgba( 0, 0, 0, 128 );
		}
		else
		{
			AABB2 bounds( center - (innerRectSize * 0.5f), center + (innerRectSize * 0.5f) );
			Rgba color( 0, 0, 0, 128 );
			theRenderer.DrawAABB2( bounds, color, true );
		}
		theRenderer.DrawTexturedRectCentered( *texture, center, innerRectSize, 0.0f, Vector2::ONE, color );
	}
}


