//-------------------------------------------------------------------------------------------------
// TheGame_Input.cpp (game)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
// 
// This source may not be used, modified, or distributed
//  without the express written permission of Brian 'Squirrel' Eiserloh.
//-------------------------------------------------------------------------------------------------
#include "pch.hpp"
#include "TheGame.hpp"



//-------------------------------------------------------------------------------------------------
// OnKeyDown
//-------------------------------------------------------------------------------------------------
void TheGame::OnKeyDown( const byte_t key )
{
	if ( _commandConsoleEnabled == true )
	{
		if ( key == VK_RETURN )
		{
			ExecuteCommand();
			_commandConsoleEnabled = false ;
			_commandBufferPosition	= 0 ;
			_commandBuffer[ _commandBufferPosition ] = 0 ;
		}
		else if ( key == VK_ESCAPE )
		{
			_commandConsoleEnabled = false ;
		}
		else if ( key == VK_BACK )
		{
			if ( _commandBufferPosition )
			{
				_commandBuffer[ --_commandBufferPosition ]	= 0 ;
			}
		}
		else 
		{
			_commandBuffer[ _commandBufferPosition ]	= key ;
			_commandBufferPosition++ ;
			_commandBuffer[ _commandBufferPosition ]	= 0 ;
		}
		return ;
	}

	if( m_gameState == GAMESTATE_ATTRACT )
	{
		if( key == VK_ESCAPE )
		{
			theApplication.RequestShutdown();
		}
		else
		{
			ChangeState( GAMESTATE_PLAYING );
		}

		return;
	}


	if( m_gameState == GAMESTATE_GAMEOVER )
	{
		if( key == VK_ESCAPE )
		{
			theApplication.RequestShutdown();
		}
		else
		{
			ChangeState( GAMESTATE_ATTRACT );
		}

		return;
	}


	if( m_gameState == GAMESTATE_YOUWIN )
	{
		if( key == VK_ESCAPE )
		{
			theApplication.RequestShutdown();
		}
		else
		{
//			ChangeState( GAMESTATE_ATTRACT );
		}

		return;
	}


	if( key == VK_ESCAPE )
	{
		theApplication.RequestShutdown();
	}
	else if( key == VK_F11 )
	{
		theRenderer.ShowFpsToggle();
	}
	else if( key == VK_TILDE )
	{
		theRenderer.ShowConsoleToggle();
	}
	// trap all the movement keys and send them to the player
	else if ( key == VK_UP || key == VK_DOWN || key == VK_LEFT || key == VK_RIGHT || key == VK_UP || key == VK_NUMPAD2 || key == VK_NUMPAD6 || key == VK_NUMPAD4 || key == VK_NUMPAD8)
	{
		m_player->OnPlayerCommandKey( key );
	}
	else if ( key == '1' || key == '2' || key == '3' || key == '4' || key == '5' || key == '6')
	{
		m_player->OnPlayerCommandKey( key );
	}
	else if( key == VK_PRIOR )
	{
		WarpToLevel( (m_currentMapNumber+1) - 1 ); // WarpToLevel is 1 based, so we add 1, then modify
	}
	else if( key == VK_NEXT )
	{
		WarpToLevel( (m_currentMapNumber+1) + 1 ); // WarpToLevel is 1 based, so we add 1, then modify
	}
	else if( key == VK_SHIFT )
	{
		m_player->ToggleNoClipMode();
	}
	else if( key == 'D' )
	{
		_mapDisplay.m_isDebugDrawingEnabled = !_mapDisplay.m_isDebugDrawingEnabled;
	}
	else if ( key == VK_RETURN )
	{
		_commandConsoleEnabled = true ;
	}
	else if( key == VK_SPACE )
	{
		m_player->OnPlayerUse();
	}
	else if( key == VK_RIGHTBRACKET )
	{
		if( GetAsyncKeyState( VK_SHIFT ) & 0x8000 )
			m_lights.m_playerRadiusScale += 0.2f / m_lights.m_mapRadiusScale;
		else
			m_lights.m_playerRadiusScale += 0.05f / m_lights.m_mapRadiusScale;
	}
	else if( key == VK_LEFTBRACKET )
	{
		if( GetAsyncKeyState( VK_SHIFT ) & 0x8000 )
			m_lights.m_playerRadiusScale -= 0.2f / m_lights.m_mapRadiusScale;
		else
			m_lights.m_playerRadiusScale -= 0.05f / m_lights.m_mapRadiusScale;
	}
	else if( key == VK_BACKSLASH )
	{
		if( m_lights.m_playerRadiusScale != 1.0f )
			m_lights.m_playerRadiusScale = 1.0f;
		else
			m_lights.m_playerRadiusScale = 1000.0f;
	}
	else if( key == 'Q' )
	{
		m_player->m_inventory.UseCurrentActiveItem();
	}
	else if( key == VK_TAB )
	{
		m_player->m_inventory.RotateCurrentActiveItem();
	}
	else
	{
		theConsole.Printf( "WARNING : OnKeyDown() for unknown key code #%d\n", key );
	}
}


//-------------------------------------------------------------------------------------------------
// OnKeyUp
//-------------------------------------------------------------------------------------------------
void TheGame::OnKeyUp( const byte_t key )
{
	// trap all the movement keys and send them to the player
	if ( key == VK_UP || key == VK_DOWN || key == VK_LEFT || key == VK_RIGHT || key == VK_UP || key == VK_NUMPAD2 || key == VK_NUMPAD6 || key == VK_NUMPAD4 || key == VK_NUMPAD8)
	{
		m_player->OnPlayerCommandKey( key );
	}
	else if( key == VK_SHIFT )
	{
		m_player->ToggleNoClipMode();
	}
}



//-------------------------------------------------------------------------------------------------
// ExecuteCommand
//-------------------------------------------------------------------------------------------------
bool TheGame::ExecuteCommand( void )
{
	theConsole.Printf( "\nExecuting: %s\n", _commandBuffer );

	if ( !strnicmp( _commandBuffer, "map", 3 ) )
	{
		char	levelName[ 255 ];
		int		levelNumber = 0 ;
		int		argsMet		= sscanf( _commandBuffer, "MAP %s %d", &levelName, &levelNumber );
		
		if ( argsMet == 2 )
		{
			WarpToLevelSet( levelName, levelNumber );
		}
		else
		{
			argsMet			= sscanf( _commandBuffer, "MAP %d", &levelNumber );
			WarpToLevel( levelNumber );
		}
	}
	else if ( strnicmp( _commandBuffer, "god", 3 ) == 0 )
	{
		theGame->m_player->ToggleGodMode();
	}
	else if ( strnicmp( _commandBuffer, "give", 4 ) == 0 )
	{
		int		itemCount = 1;
		char	itemName[ 256 ];
		int		argsMet = sscanf( _commandBuffer, "GIVE %s %d", &itemName, &itemCount );
		if( argsMet != 2 )
		{
			argsMet = sscanf( _commandBuffer, "GIVE %s", &itemName );
		}

		m_player->m_inventory.GiveItem( itemName, itemCount );
	}
	else if ( strnicmp( _commandBuffer, "remove", 6 ) == 0 )
	{
		int		itemCount = 1;
		char	itemName[ 256 ];
		int		argsMet = sscanf( _commandBuffer, "REMOVE %s %d", &itemName, &itemCount );
		if( argsMet != 2 )
		{
			argsMet = sscanf( _commandBuffer, "REMOVE %s", &itemName );
		}

		m_player->m_inventory.RemoveItem( itemName, itemCount );
	}
	else if ( strnicmp( _commandBuffer, "noclip", 6 ) == 0 )
	{
		theGame->m_player->ToggleNoClipMode();
	}
	else if ( strnicmp( _commandBuffer, "health", 6 ) == 0 )
	{
		float healthDelta = 100.0f;
		
		int argsMet = sscanf( _commandBuffer, "HEALTH %d", &healthDelta );
		theGame->m_player->AdjustHealth( healthDelta );
	}
	else if ( strnicmp( _commandBuffer, "mana", 4 ) == 0 )
	{
		float delta = 100.0f;
		
		int argsMet = sscanf( _commandBuffer, "MANA %d", &delta );
		theGame->m_player->AdjustMana( delta );
	}
	else if ( !strnicmp( _commandBuffer, "difficulty", 10 ) )
	{
		int		difficultyLevel = 1;

		int		argsMet			= sscanf( _commandBuffer, "DIFFICULTY %d", &difficultyLevel );
		difficultyLevel = ClampInt( difficultyLevel, 1, 100 );
		theGame->m_player->m_difficulty = difficultyLevel;
	}

	return false ;
}

