//-------------------------------------------------------------------------------------------------
// TheGame_Physics.cpp (game)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
// 
// This source may not be used, modified, or distributed
//  without the express written permission of Brian 'Squirrel' Eiserloh.
//-------------------------------------------------------------------------------------------------
#include "pch.hpp"
#include "TheGame.hpp"



//-------------------------------------------------------------------------------------------------
// DoBoundsCollideWithMap
//-------------------------------------------------------------------------------------------------
bool TheGame::DoBoundsCollideWithMap( const AABB2& bounds, unsigned int collisionTypeFlags )
{
	unsigned int frameNum = theApplication.GetFrameNumber();

	MapData& mapData = *theGame->m_currentMap;
	const double convertToTileCoords = 1.0 / 32.0;

	int minTileX = (int) floor( bounds.mins.x * convertToTileCoords );
	int maxTileX = (int) floor( bounds.maxs.x * convertToTileCoords );

	int minTileY = (int) floor( bounds.mins.y * convertToTileCoords );
	int maxTileY = (int) floor( bounds.maxs.y * convertToTileCoords );

	minTileX = ClampInt( minTileX, 0, MAP_TILE_SIZE_X-1 );
	maxTileX = ClampInt( maxTileX, 0, MAP_TILE_SIZE_X-1 );
	minTileY = ClampInt( minTileY, 0, MAP_TILE_SIZE_Y-1 );
	maxTileY = ClampInt( maxTileY, 0, MAP_TILE_SIZE_Y-1 );

	for( int tileX = minTileX; tileX <= maxTileX; tileX ++ )
	for( int tileY = minTileY; tileY <= maxTileY; tileY ++ )
	{
		Material& mat = mapData.GetMaterialForMapXY( tileX, tileY );
		if( mat.m_flags & collisionTypeFlags )
		{
			mapData.tileArray[ tileX ][ tileY ].m_debugFrameCountMark = frameNum;
			return true;
		}
	}

	return false;
}


//-------------------------------------------------------------------------------------------------
// IsPointWithinGivenRadiusOfPlayer
//-------------------------------------------------------------------------------------------------
bool TheGame::IsPointWithinGivenRadiusOfPlayer( const Vector2& point, float radius )
{
	Vector2 vecToPlayer = m_player->pos - point;
	float distSquared = vecToPlayer.CalcLengthSquared();

	if( distSquared < (radius * radius) )
		return true;
	else
		return false;
}
